/**
*
* $Id: CVEL-PHOTOPL.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

const double GEAR_OPERATING_SPEED = 0.25;
const double PLDOORS_OPERATING_SPEED = 0.25;
const double BRAKES_OPERATING_SPEED = 0.25;
const double HATCH_OPERATING_SPEED = 0.125;

/* make your own vessel class (named PHOTOPL here, for example)
   subclassed from COMMON_VESSEL_EX */

class PHOTOPL : public COMMON_VESSEL_EX {
public:
	PHOTOPL (OBJHANDLE hObj, int fmodel);
	~PHOTOPL();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } gear_status, pldoors_status, brakes_status, hatch_status;

	void UpdateAnimations();
	void ActivatePayloadBayDoors (DoorStatus action);
	void RevertPayloadBayDoors();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void PlaceComponents();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 

	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);

protected:
	UINT anim_pldoors;     // handle for payload door animation
	double pldoors_proc;
	MESHHANDLE		  hPHOTOPL;
	char *key;
	char *message;

};







/* basic vessel class setup */

void PHOTOPL::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void PHOTOPL::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void PHOTOPL::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void PHOTOPL::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void PHOTOPL::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void PHOTOPL::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int PHOTOPL::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new PHOTOPL (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (PHOTOPL*)vessel;
}

