/**
*
* $Id: Spiral.cpp
* (c) 2005 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* Autopilot by rjcroy
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "Spiral.h"


// 1. vertical lift component (wings and body)
// Borrowed from Mike Norman's Shuttle code

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	//                              -180 -165  -150 -135  -120 -105 -90 -75.0  -60.0  -45.0  -30.0  -15.0   0.0   15.0      30.0     45.0  60.0  75.0  90.0 105.0  120.0  135.0  150.0  165.0 180.0  
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5,  0.02, 0.655,     0.46,   0.32, 0.12, 0.05, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,     0,  0.002,  0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	// change for drag increase at high Mach:
	// Subsonic Equivalents

	double Cl0, Cl1, Cd0, Cd1, Cd2;
	if(M>=5.0) {
		Cl0 = -0.187040;
		Cl1 =  1.275600;
		Cd0 =  0.178540;
		Cd1 = -0.402900;
		Cd2 =  1.539960;
		*cl = Cl0 + Cl1*aoa;
		*cd = Cd0 + Cd1*aoa + Cd2*aoa* aoa;
	}
 	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cm = (CM[idx] + (CM[idx+1]-CM[idx])*d);
	if(M<1.25) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.04 + 0.4*oapiGetInducedDrag (*cl, 2.1, 0.82);
	}
	else if(M<5.0) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.07 + oapiGetInducedDrag (*cl, 2.1, 0.65);
// 		*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
	}
}


void VLiftCoeff2 (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	//                              -180 -165  -150 -135  -120 -105 -90 -75.0  -60.0  -45.0  -30.0  -15.0   0.0   15.0      30.0     45.0  60.0  75.0  90.0 105.0  120.0  135.0  150.0  165.0 180.0  
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5,  0.02, 0.655,     0.46,   0.32, 0.12, 0.05, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,     0,  0.002,  0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	// change for drag increase at high Mach:
	// Subsonic Equivalents

	double Cl0, Cl1, Cd0, Cd1, Cd2;
	if(M>=6.0) {
		Cl0 = -0.127040;
		Cl1 =  1.275600;
		Cd0 =  0.118540;
		Cd1 = -0.402900;
		Cd2 =  1.539960;
		*cl = Cl0 + Cl1*aoa;
		*cd = Cd0 + Cd1*aoa + Cd2*aoa* aoa;
	}
 	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cm = (CM[idx] + (CM[idx+1]-CM[idx])*d);
	if(M<1.25) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.04 + 0.4*oapiGetInducedDrag (*cl, 2.1, 0.82);
	}
	else if(M<6.0) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.07 + oapiGetInducedDrag (*cl, 2.1, 0.65);
// 		*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
	}
}

// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	
	if (M > 1.0) *cd *= 2.2;
	else *cd *= 0.6;
}

																					
void SPIRAL::InitialiseGuidance()
{
	

//	time=0.0;

	pitchinput = 0.0;
	rollinput = 0.0;
	engineignited = false;
	last_simt=0.0;
	ignition_time=0.0;
	first_guidance_pass = true;

	// rjcroy Guidance variables

	INCLINATION = 51.63 ;		// degrees

	guidance = false;			// guidance switch
	Alt_Flag = false;			// True if max launch altitude reached
	MECO_Cond = false;			// True if MECO conditions met
	First_Guidance_Pass = false;
	T_minus_10 = true;			// True at T minus 10 seconds.
	TZero = -100;				// Time Zero - Launch time
	PC_D = 0.9;					// Pitch control coefficient D for d_theta /d_t
	PC_K = 0.2;					//    "    "         "       K  "  theta
	YC_D = 0.5	;				// Yaw Control
	YC_K = 0.3;
	PC_D_FINAL = 0.9;			// Pitch control coefficients for final section.
	PC_K_FINAL = 0.6; 
	D_FINAL  = 0.8;				// Vertical velocity control coefficients
	K_FINAL = 0.8;
	ENG_THRST_2 = 0 ;		// Thrust rating of Stage 2 engine. (N)  
	i_last = 0;					// value of incliniation in previous iteration.
	Planet_Name = new char[6];
	strcpy( Planet_Name, "Earth");
	R_EARTH = 6371000;			// radius of the earth (m)
	U_EARTH = 3.986e5 * 1000 * 1000 * 1000; //( m^3/sec^2)
	Ob_EARTH = 23.45 * RAD ;	// Obliquity of the Earth (rad)
	APO_ALT = 300 ;				// Altitude of apoapsis
//	APO_ALT = 1705;

	//  inclination correction factor constant. (3580)
	ICF_K = 62.008 * INCLINATION*INCLINATION 
			- 6848.7*INCLINATION + 189807;	

	DisplayIncSP = false;		// toggle for displaying the inclination setpoint.

	NewIncSP = false ;			// True if a user has entered a new inclination SP
	PostMECO = false ;			// switch for cleaning up after MECO

	BadHeading = false;			// true if soyuz is at an azimuth not suitable for the G&CS



}

double SPIRAL::GetCPitch(double t)
{
	int i;
	if (t>met[N-1]) return cpitch[N-1];
	i=1;
	while (met[i]<t) i++;
	return cpitch[i-1]+(cpitch[i]-cpitch[i-1])/(met[i]-met[i-1])*(t-met[i-1]);
}

// rjcroy Lateral Guidance System ( called by Guidance_Routine() ).

void SPIRAL::Lateral_Guidance(VESSEL *vessel, double SimT, double SimT_Last)
{
	// Lateral Guidance System 
/*
		double hdg, diff, rate, amount, targetrate, TgtHdg;

		if (MET > 35.0) {
			targetrate = ((GetBank()*180/PI)-180)/-1.0;
			if (GetBank()<0) targetrate = -1.0*targetrate;

			if (MET > 55.0) {

				oapiGetHeading(GetHandle(),&hdg);
//				hdg = vst.arot.y * 180/PI;
				hdg *= 180/PI;
//				hdg += 180;

				GetRelativeVel (hEarth, Rel_Vel);
				GetEquPos( Longitude, Latitude, Radius );
				Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));

				oapiGetRelativeVel(hVessel, hEarth, &Rel_Vel);
				oapiGetRelativePos(hVessel, hEarth, &Rel_Pos);
				
				// Change ecliptic rel. vectors to equatorial vectors.
 				temp.x = Rel_Vel.x;
				temp.y = Rel_Vel.z;
				temp.z = Rel_Vel.y;
				veccpy( Rel_Vel, temp);
				temp.x = Rel_Pos.x;
				temp.y = Rel_Pos.z;
				temp.z = Rel_Pos.y;
				veccpy( Rel_Pos, temp);
				temp.x = Rel_Pos.x ;
				temp.y = cos(-Ob_EARTH)* Rel_Pos.y + sin(-Ob_EARTH)*Rel_Pos.z;
				temp.z =  -sin(-Ob_EARTH)* Rel_Pos.y + cos(-Ob_EARTH)*Rel_Pos.z;
				veccpy( Rel_Pos, temp);
				temp.x = Rel_Vel.x ;
				temp.y = cos(-Ob_EARTH)* Rel_Vel.y + sin(-Ob_EARTH)*Rel_Vel.z;
				temp.z =  -sin(-Ob_EARTH)* Rel_Vel.y + cos(-Ob_EARTH)*Rel_Vel.z;
				veccpy( Rel_Vel, temp);

				h_vector = crossp(Rel_Vel, Rel_Pos);
				h_abs = sqrt((h_vector.x*h_vector.x) + (h_vector.y*h_vector.y) + (h_vector.z*h_vector.z));
				i = 180 - acos(h_vector.z / h_abs) * DEG;
				i_dot = (i - i_last) / (MET - SimT_Last) ;
				i_last = i;
				ICF = Velocity / ICF_K;

				// Calculated error in inclination as output signal
				i_err = (i - INCLINATION) * (150 * ICF) + 250 * ICF * i_dot;
				if (i_err > ICF )
				{	
					i_err = ICF;
				}
				else if ( i_err < -ICF) 
				{
					i_err = -ICF;
				}
							
				double Phrad;
					//= (PadHeading-180)*PI/180;

				if ((Rel_Vel.z < 110.0) || (bgoingsouth)) {
					Phrad = PI/2 + (PI/2 - asin(cos(INCLINATION*PI/180)/cos(Latitude)));
					if ((PadHeading-180) > 180)
						Phrad = 3*PI/2 + (3*PI/2 - asin(cos(INCLINATION*PI/180)/cos(Latitude)));
					TgtHdg = Phrad*DEG;
					TgtHdg -= i_err;
					bgoingsouth = true;
				}
				else {
					Phrad =  asin(cos(INCLINATION*PI/180)/cos(Latitude));
					TgtHdg = Phrad*DEG;
					TgtHdg += i_err;
				}

				if ((TgtHdg > 90.0) || (PadHeading-180 >= 90.0))
					bgoingsouth = true;

				if ((Latitude*DEG >= i*0.98) && (fabs(i-INCLINATION)<0.02))
					bgoingsouth = true;

//				sprintf(oapiDebugString(),"Latitude %.2f  inc %.2f TGTINC %.2f going %1d",Latitude*DEG, i, INCLINATION, bgoingsouth);
//				sprintf(oapiDebugString(),"TgtHdg %.2f  inc %.2f TGTINC %.2f Phrad %.2f PadHdg %.2f tmpy %.2f",TgtHdg, i, INCLINATION, Phrad*DEG,PadHeading-180,Rel_Vel.z);

				diff = hdg - (TgtHdg);
//				if (ROLL < 90.0) diff = hdg - TgtHdg + 180 - ROLL;
//				diff = hdg - (PadHeading - 180);
				if( diff > 180.0 )
					diff -= 360.0;
				else if( diff < -180.0 )
					diff += 360.0;
				targetrate = diff/+1.0;
			}

			if( targetrate > 8.0 )
				targetrate = 8.0;
			else if( targetrate < -8.0 )
				targetrate = -8.0;


			rate = vst.vrot.y * 180/PI;
			amount = fabs (targetrate/8.0);
			if (MET > 0) amount = amount / (MET/10);

//		sprintf(oapiDebugString(),"Bank %.1f Heading %.1f TgtHdg %.1f diff %.1f Targetrate %.1f",GetBank()*180/PI,hdg,TgtHdg,diff, targetrate);

			if( fabs( rate-targetrate ) > 0.05 ) {
				if( rate < targetrate ) {
//					IncThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.05 );
//					SetThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.0 );
					yawinput = -1*amount;
				}
				else {
//					IncThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.05);
//					SetThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.0 );
					yawinput = amount;
				}
			}
			else {
//				SetThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.0 );
//				SetThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.0 );
				yawinput = 0.0;
			}
		}
*/
}

// rjcroy Guidance & Control System
void SPIRAL::Guidance_Routine(VESSEL *vessel, double SimT, double SimT_Last)
{

	hVessel = GetHandle();
	hEarth = oapiGetGbodyByName ( Planet_Name );


	if (T_minus_10)
	{
		// Record launch time
		if (TZero == 0){
			TZero = SimT + 1;
		}

		if (MET > -5 && MET < -3) {
//			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0.3);
//			LaunchClamps();
		}
		else if (MET > -3 && MET < 0) {
//			oapiSetEngineLevel(hVessel, ENGINE_MAIN, (0.2*MET + 1));
//			LaunchClamps();
			INCLINATION = 180 - acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG;
		}
		else if (MET > 0)
		{
			T_minus_10 = false;
			First_Guidance_Pass = true;
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);

			GetStatus(vst);
	
			initialgamma = vst.arot.z * 180/PI;
			launchrotate =	PadHeading - initialheading - 180 + ROLL;

			if( launchrotate < -180.0 )
				launchrotate += 360.0;
			else if( launchrotate > 180.0 )
				launchrotate -= 360.0;

		}
		sprintf(oapiDebugString(),"T - %.1f", fabs(MET));
	}		

	else if (First_Guidance_Pass) 
	{
		
		// Start the main engines
		oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);

		// Unset the first pass flag
		First_Guidance_Pass = false;
	}

	else if (!Alt_Flag) 
	{
		if (oapiGetFocusObject() == GetHandle())
			sprintf(oapiDebugString(),"T + %.1f", fabs(MET));

		if (initialgamma == 0.0)
			initialgamma = vst.arot.z * 180/PI;

		// Get correct pitch from table
		Pitch_SP=GetCPitch(MET);
		MJD_Time = oapiGetSimMJD();
		GetElements( elements, MJD_Time );
		Alt_Apoapsis = (elements.a * ( 1 + elements.e )/1000) - 6371; 
		if (Alt_Apoapsis > 100.0) {
			Pitch_SP = Pitch_SP/4.0;			// Flydown -- we've met our altitude goal
			if (ROLL < 90.0)
				if ((GetAOA()*DEG > -1.0))
					Pitch_SP += 0+0.9*GetAOA()*DEG;
			else
				if ((GetAOA()*DEG < 1.0))
					Pitch_SP += 1-0.9*GetAOA()*DEG;
		}

//		sprintf(oapiDebugString(),"Alt_Apoapsis %.1f Pitch_SP %.1f CPitch %.1f",Alt_Apoapsis, Pitch_SP, GetCPitch(MET));

		// Control to pitch setpoint
		Pitch=GetPitch(  );
		Pitch = Pitch * DEG;
		Pitch_Error = Pitch - Pitch_SP;
		if (ROLL < 90.0)
			Pitch_Error = Pitch_SP - Pitch;

		GetAngularVel( Angular_Velocity ); 
		Pitch_Error_Dot = Angular_Velocity.x * DEG; // get pitch change rate in degrees/s

		Pitch_Cntrl_Sig =  - PC_D * Pitch_Error_Dot + PC_K * Pitch_Error;
//		if ((MET > 10.0) &&(GetBank() < PI/2) && (GetBank() > -PI/2))
//			Pitch_Cntrl_Sig *= -1.0;

		if (Pitch_Cntrl_Sig > 1 )
		{
//			SetAttitudeRotLevel( 0, 1); // 0 is y-axis
			pitchinput =-1.0;
		}
		else if (Pitch_Cntrl_Sig < -1) 
		{
//			SetAttitudeRotLevel( 0, -1);
			pitchinput = +1.0;
		}
		else {
//			SetAttitudeRotLevel( 0 , Pitch_Cntrl_Sig);
			pitchinput = -Pitch_Cntrl_Sig;
		}
		

		// Heading control
//		if (ROLL > 90.0) Lateral_Guidance(this, SimT, SimT_Last);


		// Roll control
		// Adapted from Hendo's Ariane V

		if( MET > 0.0) {
				
//		vst.version = 2;
//		vst.flag = 0;
		vessel->GetStatus(vst);


		target = initialgamma - launchrotate;

		if( target > 180.0 )
			target -= 360.0;
		else if( target < -180.0 )
			target += 360.0;
					
		diff = vst.arot.z*180/PI - target;
		if( diff > 180.0 )
			diff -= 360.0;
		else if( diff < -180.0 )
			diff += 360.0;

		if (MET > 30.0) diff = 0.0;
		if (ROLL < 90.0) diff = 0.0;

		diff = GetBank()*180/PI + target;
		if (ROLL < 90.0) diff = -1.0*GetBank()*180/PI;
/*
		if ((ROLL < 90.0) && (MET < 35)) {
			oapiGetHeading(GetHandle(),&hdg);
			hdg = hdg*180/PI + 180.0;
			if (hdg < 0.0) hdg += 360.0;
			diverge = hdg-PadHeading;
			if (diverge > 35.0) diverge = 35.0;
			if (diverge < -35.0) diverge = -35.0;
			diff += diverge;
		}
*/
//		if (MET > 35) {
			diff = -1.0*GetBank()*180/PI;
			yawangle = DEG*GetSlipAngle();
			yawinput = yawangle / 15.0;
			if (ROLL > 90.0) yawinput *= -1.0;
//		}
//		sprintf(oapiDebugString(),"hdg %.1f diverge %.1f PadHeading %.1f diff %.1f ",hdg,diverge,PadHeading,diff);

//		sprintf(oapiDebugString(),"initialhdg %.1f initialgamma %.1f launchrotate %.1f target %.1f diff %.1f TGT_ %.1f",initialheading,initialgamma, launchrotate, target,diff, PadHeading);

		targetrate = diff/-1.0;
		if( targetrate > 8.0 )
			targetrate = 8.0;
		else if( targetrate < -8.0 )
			targetrate = -8.0;

		rate = vst.vrot.z * 180/PI;
		amount = fabs (targetrate/8.0);
		if (MET > 0) amount = amount / (MET/10);

//		if (amount > 0.05)
			rollinput = diff/25.0;
//		else
//			rollinput = 0.0;



		if( fabs( rate-targetrate ) > 0.05 ) {
			if( rate < targetrate ) {
				IncThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.05 );
				SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
				rollinput = -1*amount;
			}
			else {
				IncThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.05);
				SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
				rollinput = amount;
			}
		}
		else {
			SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
			SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
			rollinput = 0.0;
		}
	
//		if ((MET > 60.0) && (diff < 0.5))

//		if (MET > 60.0)
//			Lateral_Guidance(this, SimT, SimT_Last);
//		else
//			yawinput = 0.0;

//		GetAngularVel(AngularVel);

//		yawinput=(-20*AngularVel.y);		

		// END Hendo's roll control


		}

		Altitude=GetAltitude();
		a = ((APO_ALT * 1000 + R_EARTH) + (Altitude + R_EARTH)) / 2;
		e = ((APO_ALT * 1000 + R_EARTH) / a ) - 1;
		h = sqrt(U_EARTH * a * ( 1 - (e * e)));
		V_periapsis = h / (Altitude + R_EARTH);

		
		// Get current velocity
		GetRelativeVel (hEarth, Rel_Vel);
		Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));

		if ((MET > 180) && (Velocity > V_periapsis)) 
		{
			MECO_Cond = true;
			TimeMECO = SimT;
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0);
			Guidance(false);
			sprintf(oapiDebugString(), "");
			pitchinput = 0.0;
			yawinput = 0.0;
			rollinput = 0.0;
		}

		
	}

	else if (MECO_Cond && PostMECO) 
	{
		
		SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
		SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
		if ((SimT - TimeMECO) < 10) {
			sprintf(oapiDebugString(), "Guidance OFF.  Inclination (Equatorial): %.3f", i); 
		}	
		else {
			guidance = false;
			sprintf(oapiDebugString(), "");
		}
	}
}
// END Guidance_Routine()


void SPIRAL::GetInclinationSP()
{
	// Gets a new Inclination setting from the pilot
	bool AzInput (void *id, char *str, void *data);
	
//	oapiOpenInputBox ("Inclination Setpoint, e.g. 5163 = 51.63 deg (47 to 56)", IncInput, 0, 25);
	oapiOpenInputBox ("Enter launch azimuth, 0 to 90 (east) to 360", AzInput, 0, 20, (void*)this);

//	GetEquPos( Longitude, Latitude, Radius );
//	sprintf(oapiDebugString(), "Target inclination (Equatorial): %.3f", acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG); 
}


bool AzInput (void *id, char *str, void *data)
{
//	sscanf(str, "%4lf", &PadHeading);
//	PadHeading = PadHeading / 100.0;
//	PadHeading += 180.0;
/*	sscanf(str, "%4lf", &INCLINATION);
	if (INCLINATION > 4700 && INCLINATION < 5600 ) {
		INCLINATION = INCLINATION / 100;
		NewIncSP = true;
	}
*/
	return ((SPIRAL*)data)->IncInput (str);
}

bool SPIRAL::IncInput (char *rstr)
{
	sscanf(rstr, "%5lf", &PadHeading);
//	PadHeading = PadHeading / 100.0;
	PadHeading += 180.0;
	d_heading = PadHeading;
	displaytime = 0.01;
	return true;
}

// END rjcroy Guidance System

SPIRAL::SPIRAL (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hSPIRAL				= oapiLoadMeshGlobal ("SPIRAL/Spiralbody");
	hHLA				= oapiLoadMeshGlobal ("SPIRAL/HLA");
	hSPIRALBay1			= oapiLoadMeshGlobal ("SPIRAL/RBayDr");
	hSPIRALBay2			= oapiLoadMeshGlobal ("SPIRAL/LBayDr");
	hTopHatch			= oapiLoadMeshGlobal ("SPIRAL/TopHatch");	
	hEjSphere			= oapiLoadMeshGlobal ("SPIRAL/EjSphere");
	hSeat				= oapiLoadMeshGlobal ("SPIRAL/ejseat");
	hAstro				= oapiLoadMeshGlobal ("SPIRAL/USSR-eva");
	hSoyuzIntx			= oapiLoadMeshGlobal ("SPIRAL/SoyuzIntx");
	hSpiralRB2			= oapiLoadMeshGlobal ("SPIRAL/RBIntxn");
	hSpiralRB			= oapiLoadMeshGlobal ("SPIRAL/SpiralRB");
	hTailcone			= oapiLoadMeshGlobal ("SPIRAL/RBTailcone");
	hDummy1				= oapiLoadMeshGlobal ("SPIRAL/dummy1");
	hDummy2				= oapiLoadMeshGlobal ("SPIRAL/dummy2");
	hDummy3				= oapiLoadMeshGlobal ("SPIRAL/dummy3");
	hDummy4				= oapiLoadMeshGlobal ("SPIRAL/dummy4");
	hDummy5				= oapiLoadMeshGlobal ("SPIRAL/dummy5");

	tex = oapiRegisterExhaustTexture("RealExhaust");
	texLH2 = oapiRegisterExhaustTexture("exhaust_atsme");
	texRCS  = oapiRegisterExhaustTexture ("exhaust_atrcs");

	/* set the propellant handles to NULL so I can check if I need to 
	/* create them in the SetMainStage() function. */

	ph_first=NULL; 
	ph_Spiral=NULL;
	ph_escape1=NULL;
	ph_escape2=NULL;
	ph_escape3=NULL;
	ph_Jets=NULL;
	bBayOpens=true;
	bSphereJettisoned=false;
	bDeorbitFired=false;
	bThrustTerminated=false;
	bEjected=false;
	bEVA=false;
	bConeJettisoned=false;
	bBoosterMade=false;
	bControlsMade=false;
	bAutoProgrammed=false;
	gear_status=DOOR_CLOSED;
	gear_proc=0.0;
	bgear_status=DOOR_CLOSED;
	bgear_proc=0.0;
	pldoors_status=DOOR_CLOSED;
	pldoors_proc=0.0;
	wings_status=DOOR_OPEN;
	wings_proc=1.0;
	hatch_status=DOOR_CLOSED;
	hatch_proc=0.0;
	jets_status=DOOR_CLOSED;
	jets_proc=0.0;
	oldsimt = -0.01;
	Vold = _V(0,0,0);
	bAbort = false;
	AbortChecked = false;
	atchmass = 0.0;
	AutoPilot = false;
	MachPitch = 1.0;
	erasecount = 0;
	key = (char*)VirtualAlloc(NULL,256,MEM_COMMIT,PAGE_READWRITE);
	message = (char*)VirtualAlloc(NULL,256,MEM_COMMIT,PAGE_READWRITE);

	//		     			   child       pos           dir       rot      handle
	Baypos = CreateAttachment (false, _V(0,0,0.0),   _V(0,0,1), _V(0,-1,0), "X");

//	Dock0 = CreateDock(_V(0,0.9,4.8),_V(0,1,0),_V(0,0,1));

	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if( strstr( classname, "SPIRALWRB2" ) )			// Spiral with the small LOX/LH2 second stage of RB rocket
		stage = 2;
	else if( strstr( classname, "SPIRALWRB" ) )		// Spiral with RB rocket (2-stage, LOX-LH2)
		stage = 1;
	else if( strstr( classname, "SPIRALWINTX" ) ) {	// Spiral with Soyuz intersection (solid-motor escape stage)
		stage = 3;
		bBayOpens = false;
	}
	else if( strstr( classname, "SPIRAL5050" ) ) 	// Spiral with RB and HLA (termed "50-50")
		stage = 0;
	else if( strstr( classname, "HLA" ) ) {			// HLA only, no Spiral
		stage = -1;
	}
	else
		stage = 4;

	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
	RequestLoadVesselWave3(MyID,PLAYGEAR,"Sound\\SPIRAL\\gear.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYBANG,"Sound\\SPIRAL\\stg2sep.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYJETT,"Sound\\SPIRAL\\fasep.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYJET,"Sound\\SPIRAL\\jet.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYROCK,"Sound\\Vessel\\mainext.wav",BOTHVIEW_FADED_FAR);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();   
 
	MakeAnimations();

	Firstsweep = true;

}


SPIRAL::~SPIRAL()
{

	VirtualFree(key,256,MEM_DECOMMIT);
	VirtualFree(message,256,MEM_DECOMMIT);
}

void SPIRAL::MakeAnimations()
{

	anim_gear = 0.0;
	anim_bgear = 0.0;
	anim_pldoors = 0.0;
	anim_wings = 0.0;
	anim_hatch = 0.0;
	anim_jets = 0.0;
	
	anim_gear = CreateAnimation (0);
	anim_bgear = CreateAnimation (0);
	anim_pldoors = CreateAnimation(0);
	anim_hatch = CreateAnimation(0);
	anim_wings = CreateAnimation (1);
	anim_jets = CreateAnimation (0);
	anim_lelevator = CreateAnimation (0.5);
	anim_relevator = CreateAnimation (0.5);
	anim_laileron = CreateAnimation (0.5);
	anim_raileron = CreateAnimation (0.5);
	anim_rudder = CreateAnimation (0.5);


	if (stage < 1) MakeBoosterAnimations();
	if (stage < 0) return;


	static UINT LGearDoorGrp[1] = {10};
	static MGROUP_ROTATE LGearDoor (6, LGearDoorGrp, 1,
		_V(-1.038, 0.68,0.3), _V(0,0,1), (float)(-130*RAD));
	static UINT LSuptGrp[3] = {9,24,25};
	static MGROUP_ROTATE LSupt (6, LSuptGrp, 3,
		_V(-1.3, 0.174, 1.18), _V( 0.15,0, 0.9), (float)( 170*RAD));
	static UINT LGrGrp[2] = {24,25};
	static MGROUP_ROTATE LGear (6, LGrGrp, 2,
		_V(-1.3, 0.174, 1.18), _V( 1,0, 0), (float)(-110*RAD));

	static UINT RGearDoorGrp[1] = {12};
	static MGROUP_ROTATE RGearDoor (6, RGearDoorGrp, 1,
		_V( 1.038, 0.68,0.3), _V(0,0,1), (float)( 130*RAD));
	static UINT RSuptGrp[3] = {11,26,27};
	static MGROUP_ROTATE RSupt (6, RSuptGrp, 3,
		_V( 1.3, 0.174, 1.18), _V(-0.15,0, 0.9), (float)(-170*RAD));
	static UINT RGrGrp[2] = {26,27};
	static MGROUP_ROTATE RGear (6, RGrGrp, 2,
		_V( 1.3, 0.174, 1.18), _V( 1,0, 0), (float)(-110*RAD));

	static UINT LTailGearGrp[1] = {23};
	static MGROUP_ROTATE LTailGear (6, LTailGearGrp, 1,
		_V( 0.74,-0.35,-4.0), _V(0,0.0,1.0), (float)(-45*RAD));
	static UINT LTailTransGrp[1] = {23};
	static MGROUP_TRANSLATE LTailTrans (6, LTailTransGrp, 1,
		_V( 0.0,0.0,-0.30));
	static UINT RTailGearGrp[1] = {22};
	static MGROUP_ROTATE RTailGear (6, RTailGearGrp, 1,
		_V(-0.74,-0.35,-4.0), _V(0,0.0,1.0), (float)( 45*RAD));
	static UINT RTailTransGrp[1] = {22};
	static MGROUP_TRANSLATE RTailTrans (6, RTailTransGrp, 1,
		_V( 0.0,0.0,-0.30));

// Known Groups:

// 0 Fin
// 1 Rudder
// 2 R Wing
// 3 R Ail
// 4 L Wing
// 5 L Ail
// 6 LHull
// 7 SphereInnr
// 8 RCS Nubs
// 9 RGrDrLwr
// 10 RGrDr
// 11 LGrDrLwr
// 12 LGrDr 
// 13 RHull
// 14 JetDuct
// 15 InletBlock
// 16 JetEngine
// 17 RGrWell
// 18 LGrWell
// 19 BayInterior
// 20 MainEngine
// 21 SmallEngines
// 22 RA Skid
// 23 LA Skid
// 24 RF Gear Leg
// 25 RF Skid
// 26 LF Gear Leg
// 27 LF Skid

	static UINT LPLDoorGrp[1] = {0};
	static MGROUP_ROTATE LPLDoor (2, LPLDoorGrp, 1,
		_V(-0.488,0.774,0.0), _V(0,0,1), (float)( 150*RAD));
	static UINT RPLDoorGrp[1] = {0};
	static MGROUP_ROTATE RPLDoor (1, RPLDoorGrp, 1,
		_V( 0.488,0.774,0.0), _V(0,0,1), (float)(-150*RAD));
	static UINT HatchGrp[1] = {0};
	static MGROUP_ROTATE Hatch (4, HatchGrp, 1,
		_V( 0.0,1.049,1.978), _V(1,0,0), (float)( 155*RAD));
//		_V(-0.422,0.8,6.25), _V(0,0,1), (float)( 130*RAD));

	AddAnimationComponent (anim_gear, 0.05, 0.4, &LGearDoor);
	AddAnimationComponent (anim_gear, 0.4, 0.75, &LSupt);
	AddAnimationComponent (anim_gear, 0.75, 0.95, &LGear);
	AddAnimationComponent (anim_gear, 0.05, 0.4, &RGearDoor);
	AddAnimationComponent (anim_gear, 0.4, 0.75, &RSupt);
	AddAnimationComponent (anim_gear, 0.75, 0.95, &RGear);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &LTailGear);
	AddAnimationComponent (anim_gear, 0.1, 0.4, &LTailTrans);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &RTailGear);
	AddAnimationComponent (anim_gear, 0.1, 0.4, &RTailTrans);

	AddAnimationComponent (anim_pldoors, 0.1, 0.9, &LPLDoor);
	AddAnimationComponent (anim_pldoors, 0.1, 0.9, &RPLDoor);
	AddAnimationComponent (anim_hatch, 0.1, 0.9, &Hatch);


	// ***** Wing pivot animation *****

	static UINT RWingGrp[1] = {2};
	static MGROUP_ROTATE RWing2 (6, RWingGrp, 1,
		_V(-1.8,0.1,-2.7), _V(0,0,1), (float)(-50*RAD));
	static UINT RAilGrp[1] = {3};
	static MGROUP_ROTATE RAil2 (6, RAilGrp, 1,
		_V(-1.8,0.1,-2.7), _V(0,0,1), (float)( 50*RAD));
	static UINT LWingGrp[1] = {4};
	static MGROUP_ROTATE LWing2 (6, LWingGrp, 1,
		_V( 1.8,0.1,-2.7), _V(0,0,1), (float)( 50*RAD));
	static UINT LAilGrp[1] = {5};
	static MGROUP_ROTATE LAil2 (6, LAilGrp, 1,
		_V( 1.8,0.1,-2.7), _V(0,0,1), (float)(-50*RAD));
	wing1 = AddAnimationComponent (anim_wings, 0, 1, &RWing2);
	wing2 = AddAnimationComponent (anim_wings, 0, 1, &LWing2);
	AddAnimationComponent (anim_wings, 0, 1, &RAil2);
	AddAnimationComponent (anim_wings, 0, 1, &LAil2);
//	wing3 = AddAnimationComponent (anim_wings, 0, 1, &RWing2);
//	wing4 = AddAnimationComponent (anim_wings, 0, 1, &LWing2);


	// ***** Jet Duct animation *****

	static UINT JetGrp[1] = {15};
	static MGROUP_ROTATE Jet (6, JetGrp, 1,
		_V(0.0,0.56,-2.438), _V(1,0,0), (float)(-25*RAD));
	AddAnimationComponent (anim_jets, 0, 1, &Jet);


	// ***** Elevator animation *****
	static UINT LElevatorGrp[1] = {3};
	static MGROUP_ROTATE LElevator (6, LElevatorGrp, 1,
		_V(-2.15,0.83,-3.74), _V(-0.25,0.45,-0.4), (float)(-25*RAD));
	AddAnimationComponent (anim_lelevator, 0, 1, &LElevator, wing1);
	static UINT RElevatorGrp[1] = {5};
	static MGROUP_ROTATE RElevator (6, RElevatorGrp, 1,
		_V( 2.15,0.83,-3.74), _V( 0.25,0.45,-0.4), (float)( 25*RAD));
	AddAnimationComponent (anim_relevator, 0, 1, &RElevator, wing2);
/*
	// ***** Elevator trim animation *****
	static UINT LElevatorTrimGrp[1] = {3};
	static MGROUP_ROTATE LElevatorTrim (5, LElevatorTrimGrp, 1,
		_V(-2.15,0.83,-3.74), _V(-0.25,0.45,-0.4), (float)(-10*RAD));
	anim_lelevatortrim = CreateAnimation (0.5);
	AddAnimationComponent (anim_lelevatortrim, 0, 1, &LElevatorTrim, wing1);
	static UINT RElevatorTrimGrp[1] = {5};
	static MGROUP_ROTATE RElevatorTrim (5, RElevatorTrimGrp, 1,
		_V( 2.15,0.83,-3.74), _V( 0.25,0.45,-0.4), (float)( 10*RAD));
	anim_relevatortrim = CreateAnimation (0.5);
	AddAnimationComponent (anim_relevatortrim, 0, 1, &RElevatorTrim, wing2);
*/
	// ***** Aileron animation *****
	static UINT LAileronGrp[1] = {3};
	static MGROUP_ROTATE LAileron (6, LAileronGrp, 1,
		_V(-2.15,0.83,-3.74), _V(-0.25,0.45,-0.4), (float)( 20*RAD));
	AddAnimationComponent (anim_laileron, 0, 1, &LAileron, wing1);

	static UINT RAileronGrp[1] = {5};
	static MGROUP_ROTATE RAileron (6, RAileronGrp, 1,
		_V( 2.15,0.83,-3.74), _V( 0.25,0.45,-0.4), (float)( 20*RAD));
	AddAnimationComponent (anim_raileron, 0, 1, &RAileron, wing2);

	// ***** Rudder animation *****
	static UINT RudderGrp[1] = {1};
	static MGROUP_ROTATE Rudder (6, RudderGrp, 1,
		_V( 0,1.14,-4.0), _V(0,0.707,-0.707), (float)(-25*RAD));
	AddAnimationComponent (anim_rudder, 0, 1, &Rudder);



}


void SPIRAL::MakeBoosterAnimations()
{

	bBoosterMade = true;
	int meshnum = 0;


	static UINT LGearDoorGrp[1] = {13};
	static MGROUP_ROTATE LGearDoor (meshnum, LGearDoorGrp, 1,
		_V( 3.21,-5.8,-14.5), _V(0,0,1), (float)( 110*RAD));
	static UINT LGearGrp[1] = {24};
	static MGROUP_ROTATE LGear (meshnum, LGearGrp, 1,
		_V( 2.40,-5.00,-12.64), _V(1,0,0), (float)(-77*RAD));
	static UINT LGearTrkGrp[2] = {1, 25};
	static MGROUP_ROTATE LGearTrk (meshnum, LGearTrkGrp, 2,
		_V( 2.40,-5.6,-15.14), _V(1,0,0), (float)( 50*RAD));

	static UINT RGearDoorGrp[1] = {14};
	static MGROUP_ROTATE RGearDoor (meshnum, RGearDoorGrp, 1,
		_V(-3.21,-5.8,-14.5), _V(0,0,1), (float)(-110*RAD));
	static UINT RGearGrp[1] = {29};
	static MGROUP_ROTATE RGear (meshnum, RGearGrp, 1,
		_V(-2.40,-5.00,-12.64), _V(1,0,0), (float)(-77*RAD));
	static UINT RGearTrkGrp[2] = {3, 30};
	static MGROUP_ROTATE RGearTrk (meshnum, RGearTrkGrp, 2,
		_V(-2.40,-5.6,-15.14), _V(1,0,0), (float)( 50*RAD));

	static UINT NoseGearDoorGrp[1] = {16};
	static MGROUP_ROTATE NoseGearDoor (meshnum, NoseGearDoorGrp, 1,
		_V(-0.325,-4.45,13.0), _V(0.04,0.1,1), (float)(-100*RAD));
	static UINT NoseGearGrp[2] = {0, 23};
	static MGROUP_ROTATE NoseGear (meshnum, NoseGearGrp, 2,
		_V(0.0,-4.17,11.062), _V(1,0,0), (float)( 99*RAD));

	static UINT NoseDroopGrp[3] = {8, 28, 32};
	static MGROUP_ROTATE NoseDroop (meshnum, NoseDroopGrp, 3,
		_V(0.0,-4.425,10.9), _V(1,0,0), (float)(7*RAD));
	static UINT VentralFinGrp[1] = {6};
	static MGROUP_ROTATE VentralFin (meshnum, VentralFinGrp, 1,
		_V(0.0,-6.6,-20.0), _V(0,0,1), (float)(-90*RAD));

// Known Groups:

// 0 Nose Tires
// 1 L Tires
// 2 View Blocks
// 3 R Tires
// 4 L Ail
// 5 L Wing
// 6 Ventral Fin
// 7 L Endplate
// 8 DroopSnoot
// 9 BoatTail
// 10 R Endplate
// 11 R Wing
// 12 R Ail
// 13 L Gr Dr
// 14 R Gr Dr
// 15 JetDuct
// 16 Nose Gr Dr
// 17 L Rudder
// 18 R Rudder
// 19 Nose Fairing
// 20 Hull
// 21 Inlet Comb
// 22 Engines
// 23 Nose Gear
// 24 L Gear Strut
// 25 L Gear Truck
// 26 L Gear Box
// 27 R Gear Box
// 28 Nose gear box
// 29 R Gear Strut
// 30 R Gear Truck
// 31 CptInner
// 32 SnootInner
// 33 Window



	Gear11 = AddAnimationComponent (anim_bgear, 0.05, 0.4, &LGearDoor);
	Gear12 = AddAnimationComponent (anim_bgear, 0.4, 0.9, &LGear);
	Gear13 = AddAnimationComponent (anim_bgear, 0.4, 0.9, &LGearTrk,Gear12);
	Gear14 = AddAnimationComponent (anim_bgear, 0.05, 0.4, &RGearDoor);
	Gear15 = AddAnimationComponent (anim_bgear, 0.4, 0.9, &RGear);
	Gear16 = AddAnimationComponent (anim_bgear, 0.4, 0.9, &RGearTrk,Gear15);
	Gear17 = AddAnimationComponent (anim_bgear, 0.4, 0.9, &NoseGear);
	Gear18 = AddAnimationComponent (anim_bgear, 0.1, 0.7, &NoseGearDoor);
	Gear19 = AddAnimationComponent (anim_bgear, 0.1, 0.9, &NoseDroop);
	Gear1A = AddAnimationComponent (anim_bgear, 0.1, 0.9, &VentralFin);

	// ***** Elevator animation *****
	static UINT LElevatorGrp[1] = {12};
	static MGROUP_ROTATE LElevator (meshnum, LElevatorGrp, 1,
		_V(-5.0,-4.55,-16.8), _V(-1,0,-0.32), (float)(-25*RAD));
	Elev11 = AddAnimationComponent (anim_lelevator, 0, 1, &LElevator);
	static UINT RElevatorGrp[1] = {4};
	static MGROUP_ROTATE RElevator (meshnum, RElevatorGrp, 1,
		_V( 5.0,-4.55,-16.8), _V(-1,0, 0.32), (float)(-25*RAD));
	Elev12 = AddAnimationComponent (anim_relevator, 0, 1, &RElevator);

	// ***** Elevator trim animation *****
	static UINT LElevatorTrimGrp[1] = {12};
	static MGROUP_ROTATE LElevatorTrim (meshnum, LElevatorTrimGrp, 1,
		_V(-5.0,-4.55,-16.8), _V(-1,0,-0.32), (float)(-10*RAD));
	Elev13 = AddAnimationComponent (anim_lelevatortrim, 0, 1, &LElevatorTrim);
	static UINT RElevatorTrimGrp[1] = {4};
	static MGROUP_ROTATE RElevatorTrim (meshnum, RElevatorTrimGrp, 1,
		_V( 5.0,-4.55,-16.8), _V(-1,0, 0.32), (float)(-10*RAD));
	Elev14 = AddAnimationComponent (anim_relevatortrim, 0, 1, &RElevatorTrim);

	// ***** Rudder animation *****
	static UINT RRudderGrp[1] = {17};
	static MGROUP_ROTATE RRudder (meshnum, RRudderGrp, 1,
		_V( 11.83,-3.25,-20.338), _V( 0,0.9,-0.5), (float)(-30*RAD));
	static UINT LRudderGrp[1] = {18};
	static MGROUP_ROTATE LRudder (meshnum, LRudderGrp, 1,
		_V(-11.83,-3.25,-20.338), _V( 0,0.9,-0.5), (float)(-30*RAD));
	Rudd11 = AddAnimationComponent (anim_rudder, 0, 1, &RRudder);
	Rudd12 = AddAnimationComponent (anim_rudder, 0, 1, &LRudder);

	// ***** Aileron animation *****

	static UINT RAileronGrp[1] = {12};
	static MGROUP_ROTATE RAileron (meshnum, RAileronGrp, 1,
		_V(-5.0,-4.55,-16.8), _V(-1,0,-0.32), (float)( 30*RAD));
	Rudd13 = AddAnimationComponent (anim_raileron, 0, 1, &RAileron);

	static UINT LAileronGrp[1] = {4};
	static MGROUP_ROTATE LAileron (meshnum, LAileronGrp, 1,
		_V( 5.0,-4.55,-16.8), _V(-1,0, 0.32), (float)(-30*RAD));
	Rudd14 = AddAnimationComponent (anim_laileron, 0, 1, &LAileron);


}

void SPIRAL::DelBoosterAnimations()
{

	bBoosterMade = false;

	
	DelAnimationComponent (anim_bgear,Gear11);
	DelAnimationComponent (anim_bgear,Gear12);
	DelAnimationComponent (anim_bgear,Gear13);
	DelAnimationComponent (anim_bgear,Gear14);
	DelAnimationComponent (anim_bgear,Gear15);
	DelAnimationComponent (anim_bgear,Gear16);
	DelAnimationComponent (anim_bgear,Gear17);
	DelAnimationComponent (anim_bgear,Gear18);
	DelAnimationComponent (anim_bgear,Gear19);
	DelAnimationComponent (anim_bgear,Gear1A);
	DelAnimationComponent (anim_lelevator,Elev11);
	DelAnimationComponent (anim_relevator,Elev12);
	DelAnimationComponent (anim_lelevatortrim,Elev13);
	DelAnimationComponent (anim_relevatortrim,Elev14);
	DelAnimationComponent (anim_rudder,Rudd11);
	DelAnimationComponent (anim_rudder,Rudd12);
	DelAnimationComponent (anim_raileron,Rudd13);
	DelAnimationComponent (anim_laileron,Rudd14);

}

void SPIRAL::MakeAirfoils()
{
	ClearAirfoilDefinitions();
	ClearVariableDragElements();

	if (stage > 0) {

		CreateVariableDragElement (&gear_proc, 0.8, _V(0, -1, 0));
		CreateVariableDragElement (&MachPitch, 1.8, _V(0, 18.0,-4.0));


//					align,         attack pt,   coeff func, c, S,  AR
	CreateAirfoil (LIFT_VERTICAL, _V(0,0,-0.2), VLiftCoeff, 7, 66, 1.55);
	// wing and body lift+drag components

	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-3.5), HLiftCoeff, 4, 2, 3.625);
	// vertical stabiliser and body lift and drag components

	}
	else {

		CreateVariableDragElement (&gear_proc, 1.8, _V(0, -1, 0));


	//                                alignment      position       func    chord S   AR   
		BstrWing = CreateAirfoil2 (LIFT_VERTICAL, _V(0,0,-0.15), VLiftCoeff2, 18, 240, 3.35);
		// wing and body lift+drag components

		BstrRudder = CreateAirfoil2 (LIFT_HORIZONTAL, _V(0,0,-14.5), HLiftCoeff, 8, 38, 2.6);
		// vertical stabiliser and body lift and drag components

	}

	MakeControlSurfaces();
}


void SPIRAL::MakeControlSurfaces()
{

//  Aargh!  Can't delete control surfaces
//  Need to make sure we declare once and only once

	if (bControlsMade) return;
	bControlsMade = true;

	CreateControlSurface (AIRCTRL_ELEVATOR,     1.9,  0.8, _V(  3.2,0,-4.67), AIRCTRL_AXIS_XPOS, anim_lelevator);
	CreateControlSurface (AIRCTRL_ELEVATOR,     1.9,  0.8, _V( -3.2,0,-4.67), AIRCTRL_AXIS_XPOS, anim_relevator);
	CreateControlSurface (AIRCTRL_RUDDER,       2.4,  0.7, _V(   0, 0,-3.75), AIRCTRL_AXIS_YPOS, anim_rudder);
	CreateControlSurface (AIRCTRL_AILERON,      2.1, -0.85, _V(-3.2,0,-4.58), AIRCTRL_AXIS_XPOS, anim_raileron);
	CreateControlSurface (AIRCTRL_AILERON,      2.1, -0.85, _V( 3.2,0,-4.58), AIRCTRL_AXIS_XNEG, anim_laileron);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 1.9,  0.8, _V(  3.2,0,-4.67), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 1.9,  0.8, _V( -3.2,0,-4.67), AIRCTRL_AXIS_XPOS);

}




void SPIRAL::UpdateAnimations()
{	

	// animate booster landing gear
	if (stage < 1) {
		if (bgear_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * GEAR_OPERATING_SPEED;
			if (bgear_status == DOOR_CLOSING) {
				if (bgear_proc > 0.0)
					bgear_proc = max (0.0, bgear_proc-da);
				else {
					bgear_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (bgear_proc < 1.0)
					bgear_proc = min (1.0, bgear_proc+da);
				else {
					bgear_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_bgear, bgear_proc);
		}
	return;
	}

	// animate landing gear
	if (gear_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * GEAR_OPERATING_SPEED;
		if (gear_status == DOOR_CLOSING) {
			if (gear_proc > 0.0)
				gear_proc = max (0.0, gear_proc-da);
			else {
				gear_status = DOOR_CLOSED;
			}
		} else  { // door opening
			if (gear_proc < 1.0)
				gear_proc = min (1.0, gear_proc+da);
			else {
				gear_status = DOOR_OPEN;
			}
		}
		SetAnimation (anim_gear, gear_proc);
	}

	// animate payload bay doors
	if (pldoors_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * PLDOORS_OPERATING_SPEED;
		if (pldoors_status == DOOR_CLOSING) {
			if (pldoors_proc > 0.0)
				pldoors_proc = max (0.0, pldoors_proc-da);
			else {
				pldoors_status = DOOR_CLOSED;
			}
		} else  { // door opening
			if (pldoors_proc < 1.0)
				pldoors_proc = min (1.0, pldoors_proc+da);
			else {
				pldoors_status = DOOR_OPEN;
			}
		}
		SetAnimation (anim_pldoors, pldoors_proc);
	}

	// animate wings
	if (wings_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * WINGS_OPERATING_SPEED;
		if (wings_status == DOOR_CLOSING) {
			if (wings_proc > 0.0)
				wings_proc = max (0.0, wings_proc-da);
			else {
				wings_status = DOOR_CLOSED;
			}
		} else  { // door opening
			if (wings_proc < 1.0)
				wings_proc = min (1.0, wings_proc+da);
			else {
				wings_status = DOOR_OPEN;
			}
		}
		SetAnimation (anim_wings, wings_proc);
	}

	// animate jet duct
	if (jets_status >= DOOR_CLOSING) {
		double da = oapiGetSimStep() * JETS_OPERATING_SPEED;
		if (jets_status == DOOR_CLOSING) {
			if (jets_proc > 0.0)
				jets_proc = max (0.0, jets_proc-da);
			else {
				jets_status = DOOR_CLOSED;
			}
		} else  { // door opening
			if (jets_proc < 1.0)
				jets_proc = min (1.0, jets_proc+da);
			else {
				jets_status = DOOR_OPEN;
				SwitchToJets();
			}
		}
		SetAnimation (anim_jets, jets_proc);
	}

	// animate top hatch
	if (!bEjected)
		if (hatch_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * HATCH_OPERATING_SPEED;
			if (hatch_status == DOOR_CLOSING) {
				if (hatch_proc > 0.0)
					hatch_proc = max (0.0, hatch_proc-da);
				else {
					hatch_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (hatch_proc < 1.0)
					hatch_proc = min (1.0, hatch_proc+da);
				else {
					hatch_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_hatch, hatch_proc);
		}

}

void SPIRAL::ActivateLandingGear (DoorStatus action)
{
	gear_status = action;
}

void SPIRAL::RevertLandingGear ()
{
	ActivateLandingGear (gear_status == DOOR_CLOSED || gear_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void SPIRAL::ActivateBoosterLandingGear (DoorStatus action)
{
	bgear_status = action;
}

void SPIRAL::RevertBoosterLandingGear ()
{
	ActivateBoosterLandingGear (bgear_status == DOOR_CLOSED || bgear_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void SPIRAL::ActivatePayloadBayDoors (DoorStatus action)
{
	pldoors_status = action;
}

void SPIRAL::RevertPayloadBayDoors ()
{
	ActivatePayloadBayDoors (pldoors_status == DOOR_CLOSED || pldoors_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void SPIRAL::ActivateHatch (DoorStatus action)
{
	hatch_status = action;
}

void SPIRAL::RevertHatch ()
{
	ActivateHatch (hatch_status == DOOR_CLOSED || hatch_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void SPIRAL::ActivateWings (DoorStatus action)
{
	wings_status = action;
}

void SPIRAL::RevertWings ()
{
	ActivateWings (wings_status == DOOR_CLOSED || wings_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void SPIRAL::ActivateJets (DoorStatus action)
{
	jets_status = action;
}

void SPIRAL::RevertJets ()
{
	ActivateJets (jets_status == DOOR_CLOSED || jets_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}


void SPIRAL::SwitchToJets ()
{


	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 325.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
//		0, 1.25, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};

	SetDefaultPropellantResource (ph_Jets); 

	DelThruster(th_first[0]);
	th_first[0] = CreateThruster ( _V(0,0.5,-4.3), _V(0,-0.08,1), THRUST_JET_VAC, ph_Jets, ISP_JET_VAC, ISP_JET_SL);

	thg_first = CreateThrusterGroup (th_first, 1, THGROUP_MAIN);
//	AddExhaust (th_first[0], 3.0, 0.3, tex);

	AddExhaustStream (th_first[0], _V(0,0.5,-5.5), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

}


void SPIRAL::AirBreathingEngine()
// Thanks to David Hopkins for publishing his AirBreathingEngine source.  
{//=========================================================
double alt2;
double eas2;
double cf1, cf2;
double ActualThrust;


alt2 = GetAltitude();
eas2 = GetAirspeed ();

alt2 = alt2*3.28;
eas2 = eas2*2.236;

cf1 = 1.67e-6*(eas2*eas2)-8.8789e-4*(eas2)+0.9977;
cf2 = 3.0e-10*(alt2*alt2)-3.0000e-5*(alt2)+0.9934;

// Nasty kludge to keep the thrust from increasing dramatically above 50kft
if (alt2 > 50000)
	cf2 = 0.25*(1-((alt2-50000)*.0001));
if (cf2 < 0) cf2 = 0;

ActualThrust = cf1*cf2*THRUST_JET_VAC;

SetThrusterMax0 (th_first[0],ActualThrust);
};


void SPIRAL::AirBreathingEngine2()
// Thanks to David Hopkins for publishing his AirBreathingEngine source.  
{//=========================================================
double alt2;
double eas2;
double cf1, cf2;
double ActualThrust;
double DesiredThrust;
double RocketFraction;
double ActualISP;


alt2 = GetAltitude();
eas2 = GetAirspeed ();

alt2 = alt2*3.28;
alt2 = alt2*0.62;		// a kludge that keeps the "airbreathing" engine operating at higher altitude
eas2 = eas2*2.236;

cf1 = 1.67e-6*(eas2*eas2)-8.8789e-4*(eas2)+0.9977;	
cf2 = 3.0e-10*(alt2*alt2)-3.0000e-5*(alt2)+0.9934;

// Nasty kludge to keep the thrust from increasing dramatically above 100kft
if (alt2 > 50000)
	cf2 = 0.25*(1-((alt2-50000)*.0001));
if (cf2 < 0) cf2 = 0;

ActualThrust = cf1*cf2*THRUST_HLA_VAC;
DesiredThrust = GetThrusterGroupLevel(thg_first)*THRUST_HLA_VAC;
RocketFraction = (DesiredThrust - ActualThrust)/DesiredThrust;
if (RocketFraction < 0) RocketFraction = 0.0;
if ((alt2 > 30000) && (RocketFraction < 0.2)) RocketFraction = 0.2;		// a little rocket power to simulate ramjet
ActualISP = (RocketFraction*ISP_HLA_VAC + (1-RocketFraction)*ISP_JET_VAC);

//sprintf(oapiDebugString(),"ActualThrust %.4f DesiredThrust %.4f RocketFraction %.4f Isp %.4f",ActualThrust, DesiredThrust, RocketFraction, ActualISP);

SetThrusterIsp(th_first[0],ActualISP);
SetThrusterIsp(th_first[1],ActualISP);
SetThrusterIsp(th_first[2],ActualISP);
SetThrusterIsp(th_first[3],ActualISP);

double ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN)*(1-RocketFraction);


		if(ThrusterActive>0.0f)
			{
			// we calculate the volume at wich it must be played
			double volume=110+(145*ThrusterActive);
			double freq=11050+(9000*ThrusterActive);
			PlayVesselWave3(MyID,PLAYJET,LOOP,volume,freq); // play the sound
			}
		else if(IsPlaying3(MyID,PLAYJET))
			{
			// else shut down the sound if it's playing
			StopVesselWave3(MyID,PLAYJET);
			}

		ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN)*(RocketFraction);
		if(ThrusterActive>0.0f)
			{
			// we calculate the volume at wich it must be played
			double volume=110+(145*ThrusterActive);
			double freq=11050+(9000*ThrusterActive);
			PlayVesselWave3(MyID,PLAYROCK,LOOP,volume,freq); // play the sound
			}
		else if(IsPlaying3(MyID,PLAYROCK))
			{
			// else shut down the sound if it's playing
			StopVesselWave3(MyID,PLAYROCK);
			}

//SetThrusterMax0 (th_first[0],ActualThrust);
};

void SPIRAL::SetGearParameters (double state)
{

	double geary = ((1.0-state)*0.35);
	if (stage < 1)
		SetTouchdownPoints (_V(0,-7.3+3*geary,14.0), _V(-9.15,-7.97-geary,-12.5), _V(9.15,-7.97+geary,-12.5));
	else
		SetTouchdownPoints (_V(0,-1.0+1.5*geary,14.0), _V(-9.15,-1.0+geary,-3.0), _V(9.15,-1.0+geary,-3.0));
	if (state == 1.0 && !BrakesOn) {
		SetSurfaceFrictionCoeff (0.1, 0.4);
	} else {
		SetSurfaceFrictionCoeff (0.1, 0.4);		// Skids; no brakes!
	}
}


void SPIRAL::JettisonSphere ()
{
	bSphereJettisoned = true;
	VESSELSTATUS vs;
	GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	vs.status=0;
	VECTOR3 ofs = ENTRYSHIELD_OFS;
	VECTOR3 vel = _V(0,2.0,0.0);
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, GetName()); strcat (name, "-Sphere");
	VESSEL::Create (name, "SPIRAL/SPIRAL_EjectSphere", vs);
	OBJHANDLE hvessel;
	hvessel=oapiGetVesselByName(name);
	if (hvessel != 0) 
		oapiSetFocusObject(hvessel);
	SetMainStage();
	PlayVesselWave3(MyID,PLAYJETT);
}

void SPIRAL::JettisonCone ()
{
	bConeJettisoned = true;
	VESSELSTATUS vs;
	GetStatus (vs);
	VECTOR3 ofs = TAILCONE_OFS;
	VECTOR3 vel = _V(0,0.0,-1.0);
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, GetName()); strcat (name, "-Tailcone");
	VESSEL::Create (name, "SPIRAL/Cone", vs);
	SetMainStage();
	PlayVesselWave3(MyID,PLAYJETT);
}

void SPIRAL::SeparateEVA ()
{
	VESSELSTATUS vs;
	GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	vs.status=0;
	VECTOR3 ofs = ASTRO_OFS;
	Local2Rel (ofs, vs.rpos);
	char name[256];
	strcpy (name, GetName()); strcat (name, "-PLT");
	oapiCreateVessel (name, "SPIRAL/eva", vs);
}


void SPIRAL::TimeStep(  double simt )
{
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */
	ZeroMemory (key,256);
	ZeroMemory (message,256);

	if (stage > 1) {
		if( CheckCentralMessage(key, message) ) {
			CheckCentralMessage();
			if (( strstr(key, "ON" ) ) && (stage == 0) && (!AutoPilot) && ( strstr(message, "AUTOPILOT" ) )) {
	//			sprintf(oapiDebugString(),"Autopilot in continued operation");
				AutoPilot = true;
				ActivateNavmode(NAVMODE_PROGRADE);
				SetThrusterGroupLevel (thg_first,1.0);
			}
			while( CheckCentralMessage() ) ;
				SetMainStage();
		}

		if (AutoPilot) SetThrusterGroupLevel (thg_first,1.0);

		if (AutoPilot) {
			GetShipAirspeedVector(Rel_Vel);
			Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));
			Altitude=GetAltitude();
			a = ((APO_ALT * 1000 + R_EARTH) + (Altitude + R_EARTH)) / 2;
			e = ((APO_ALT * 1000 + R_EARTH) / a ) - 1;
			h = sqrt(U_EARTH * a * ( 1 - (e * e)));
			V_periapsis = h / (Altitude + R_EARTH);
					
			if (Velocity > V_periapsis) 
			{
	//			sprintf(oapiDebugString(),"Autopilot Off");
				AutoPilot = false;
				ActivateNavmode(NAVMODE_KILLROT);
				SetThrusterGroupLevel (thg_first,0.0);
			}
		}
	}

	erasecount++;						//  Every once in a while, check for spawned EVA's that are not focus-enabled,
	if (erasecount > 60) {				//  and delete them.
		erasecount = 0;
		char name[256];
			strcpy (name, GetName()); strcat (name, "-PLT");
			OBJHANDLE hvessel=oapiGetVesselByName(name);
			if (hvessel != 0) {
				VESSEL *remv = oapiGetVesselInterface(hvessel);
				if (remv->GetEnableFocus() == false)
					oapiDeleteVessel(hvessel);
			}
	}

// Check to see if the EVA guy is back inside.  If so, start drawing the pilot in his seat again
	if (bEVA) {
		char name[256];
		OBJHANDLE hvessel;
		strcpy (name, GetName()); strcat (name, "-PLT");
		hvessel=oapiGetVesselByName(name);
		if (hvessel == 0) {
			bEVA = false;
			SetMainStage();
		}
	}

	if (!bAutoProgrammed)
		ProgramAuto();
	if (!bControlsMade)
		MakeControlSurfaces();

/*	dt = simt - oldsimt;
	oldsimt = simt;
	if ((displaytime > 0) && (oapiGetFocusObject() == GetHandle())) displaytime += dt;

	if ((!guidance) && (displaytime < 5) && (stage == 0) && (oapiGetFocusObject() == GetHandle()))
	{
		GetEquPos( Longitude, Latitude, Radius );
		sprintf(oapiDebugString(), "Target inclination (Equatorial): %.3f", (180 - acos(sin(d_heading*PI/180)*cos(Latitude))*DEG));
	}

	if (!guidance && !MECO_Cond ) {
		if(!NewIncSP) {
			// Set the Inclination setpoint for the guidance computer
			GetEquPos( Longitude, Latitude, Radius );
				INCLINATION = acos(Clairaut_Constant)*DEG;
		}
	}	
*/
	//Check if guidance is on and if so, call
	if (guidance)
	{
		{
			// Calculate MET
			if (TZero == -100) 
			{
				MET = -5 ;
				TZero = 0;
			}
			else 
			{
				MET = simt - TZero;
			}
			// Call the guidance routine
			Guidance_Routine(this, simt, SimT_Last);
			SimT_Last = MET;
		}
	}

// Fix the amount of hypersonic pitch-up
	MachPitch = GetMachNumber() - 5.15;
	if (MachPitch > 1.0) MachPitch = 1.0;
	if ((MachPitch < 0.0) || (wings_status==DOOR_CLOSED)) MachPitch = 0.0;
	if ((stage < 4) && (MachPitch > 0.25)) MachPitch = 0.25;  // Draggy cone stabilizes a bit
	if (stage < 2) MachPitch = 0.0;  // RB strakes stabilize the ship


	if ((stage == 3) && (!AbortChecked)) {
		AbortChecked = true;
		GetShipAirspeedVector(Rel_Vel);
		Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));
		if (Velocity < 4500.0)
			bAbort = true;
	}

/*	if ((stage < 2) && (jets_status == DOOR_CLOSED))					// Throttle back to maintain 3.5 G's
		if (((GetThrusterGroupLevel(THGROUP_MAIN)*THRUST_FIRST_VAC*numengines)/GetMass()) > 3.5*G)
			SetThrusterGroupLevel(THGROUP_MAIN,(GetThrusterGroupLevel(THGROUP_MAIN)*0.98));
*/
	if ((stage==4) && (GetMachNumber() < 5.15) && (GetAtmPressure() > 100.0) && (wings_status==DOOR_OPEN))
		RevertWings();
	if ((stage==4) && (GetMachNumber() < 0.98) && (GetAtmPressure() > 100.0) && (jets_status==DOOR_CLOSED))
		RevertJets();
	
	if (stage < 1) 
		AirBreathingEngine2();

	if ((stage==4) && (jets_status==DOOR_OPEN)) {
		AirBreathingEngine();
		double ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN);
		if(ThrusterActive>0.0f)
			{
			// we calculate the volume at wich it must be played
			double volume=110+(145*ThrusterActive);
			double freq=11050+(9000*ThrusterActive);
			PlayVesselWave3(MyID,PLAYJET,LOOP,volume,freq); // play the sound
			}
		else if(IsPlaying3(MyID,PLAYJET))
			{
			// else shut down the sound if it's playing
			StopVesselWave3(MyID,PLAYJET);
			}
	}

	UpdateAnimations();
	if (stage < 3) VectorThrust();
	if (stage < 1) SetGearParameters(bgear_proc);
	else SetGearParameters(gear_proc);
	if (bAbort) {
		if (stage==3) {
			SetThrusterGroupLevel(thg_first2,1.0);
			for (i=0; i < 12; i++)
				SetThrusterLevel (th_first[i],1.0);
		}
	if (GetThrusterLevel(th_first[0]) > 0.0)
		PlayVesselWave3(MyID,PLAYROCK,LOOP,255,11040); // play the sound
	else if(IsPlaying3(MyID,PLAYROCK))
		StopVesselWave3(MyID,PLAYROCK);
	}

	if ((stage==3) && (!bAbort)) {
		if ((GetThrusterGroupLevel (thg_first) == 0.0) && (bDeorbitFired) && (!bThrustTerminated))
			bThrustTerminated = true;
		if (GetThrusterGroupLevel (thg_first) > 0.05) {
			bDeorbitFired = true;
			if (GetPropellantMass(ph_escape2) == 0.0)
				Burn3 = true;
			if (GetPropellantMass(ph_escape1) == 0.0)
				Burn2 = true;
			Burn1 = true;
//			bThrustTerminated = false;
		}
	}

	if ((bDeorbitFired) && (!bAbort) && (stage==3)) {     // Act as a solid motor (burn to depletion)
//		SetThrusterGroupLevel(thg_first,0.0);
		for (i=0; i < 12; i++)
			SetThrusterLevel (th_first[i],0.0);
		if (Burn1) {
			SetThrusterLevel (th_first[0],1.0); 
			SetThrusterLevel (th_first[1],1.0); 
			SetThrusterLevel (th_first[2],1.0); 
			SetThrusterLevel (th_first[3],1.0); 
			Burn1 = true;
		}
		if (Burn2) {
			//((GetPropellantMass(ph_escape2) > 0.0) || ((!bThrustTerminated) && (GetPropellantMass(ph_escape2) < 0.9*ESCAPE_PROP_MASS))) {
			SetThrusterLevel (th_first[4],1.0); 
			SetThrusterLevel (th_first[5],1.0); 
			SetThrusterLevel (th_first[6],1.0); 
			SetThrusterLevel (th_first[7],1.0); 
			Burn2 = true;
		}
		if (Burn3) {
			//((GetPropellantMass(ph_escape3) > 0.0) || ((!bThrustTerminated) && (GetPropellantMass(ph_escape3) < 0.9*ESCAPE_PROP_MASS))) {
			SetThrusterLevel (th_first[8],1.0); 
			SetThrusterLevel (th_first[9],1.0);
			SetThrusterLevel (th_first[10],1.0); 
			SetThrusterLevel (th_first[11],1.0); 
			Burn3 = true;
		}
		if ((Burn1) || (Burn2) || (Burn3))
			PlayVesselWave3(MyID,PLAYROCK,LOOP,255,11040); // play the sound
		else if(IsPlaying3(MyID,PLAYROCK))
			StopVesselWave3(MyID,PLAYROCK);

	}


	if (Firstsweep) {
		AttachSweep();
		if (stage==3)
			if ((GetPropellantMass(ph_escape1) < 1.1) && (GetPropellantMass(ph_Spiral) < 1.1))
			{
			SetPropellantMass(ph_escape1,ESCAPE_PROP_MASS);
			SetPropellantMass(ph_escape2,ESCAPE_PROP_MASS);
			SetPropellantMass(ph_escape3,ESCAPE_PROP_MASS);
			SetPropellantMass(ph_Spiral,RCS_PROP_MASS);
			}
		Firstsweep = false;
	}

	if ((stage == 1) && (!bConeJettisoned) && (GetThrusterLevel(th_first[0]) > 0.0))
		JettisonCone();

	if (stage == 2) 
		if (GetPropellantMass(ph_first) == 0) {
			JettisonPart();
			SetMainStage();
			Guidance( false );
		}

	if (stage == 1) 
		if (GetPropellantMass(ph_first) == 0) {
			JettisonPart();
			SetMainStage();
			SetThrusterGroupLevel (THGROUP_MAIN,1.0);
		}

	if (stage == 0)
		if (GetPropellantMass(ph_booster) == 0) {
			JettisonPart();
			AttachSweep();

			/* IMPORTANT */
			/* Always re-set the stage after any kind of change */
			SetMainStage();
			SetThrusterGroupLevel (THGROUP_MAIN,1.0);
		}

	if (Firstsweep) {
		AttachSweep();
		Firstsweep = false;
	}
}


void SPIRAL::VectorThrust()
{

	if (Firstsweep)
		return;

	if (oapiGetTimeAcceleration() > 10) return;

		tempP = GetManualControlLevel(THGROUP_ATT_PITCHDOWN, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_PITCHUP, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

		tempY = GetManualControlLevel(THGROUP_ATT_YAWLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_YAWRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

		tempR = GetManualControlLevel(THGROUP_ATT_BANKLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_BANKRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

		GetAngularVel(AngularVel);

		tempP+=(2*AngularVel.x);			// Bias the thrust vector AGAINST the angular velocity a bit
		tempY+=(-2*AngularVel.y);			// This will damp angular velocity, preventing loss of control
		tempR+=(2*AngularVel.z);
	

		tempP+=pitchinput;
		tempY+=yawinput;
		tempR+=rollinput;
	


		if (stage < 1) {
			SetThrusterDir(th_first[0],_V(0.012*tempY,0.012*(tempP-2*tempR),1));
			SetThrusterDir(th_first[1],_V(0.012*tempY,0.012*(tempP+2*tempR),1));
		}
		else if (stage == 1) {
			SetThrusterDir(th_first[0],_V(0.012*tempY,0.012*(tempP),1));
		}
		else if (stage == 2) {
			SetThrusterDir(th_first[0],_V(0.012*tempY,0.012*(tempP),1));
		}

}

/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */


void SPIRAL::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	SetMainStage(); // this is a good place to ask for the initial setup.
}


void SPIRAL::SetClassCaps(  FILEHANDLE cfg )
{

	CheckCentralMessage();

	SetMainStage();
}

void SPIRAL::SaveState( FILEHANDLE scn )
{
	char cbuf[256];

	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */


//	Undock(ALLDOCKS);

	oapiWriteScenario_int (scn, "CONFIGURATION", stage);
	oapiWriteScenario_int (scn, "BAY_OPENS", bBayOpens);
	oapiWriteScenario_int (scn, "SPHEREGONE", bSphereJettisoned);
	oapiWriteScenario_int (scn, "CONEGONE", bConeJettisoned);
	oapiWriteScenario_int (scn, "EJECTED", bEjected);
	oapiWriteScenario_int (scn, "EVA", bEVA);
	oapiWriteScenario_int (scn, "DEORBIT FIRED", bDeorbitFired);
	oapiWriteScenario_int (scn, "THRUST TERMINATED", bThrustTerminated);
	sprintf (cbuf, "%d %0.4f", bgear_status, bgear_proc);
	oapiWriteScenario_string (scn, "BOOSTERGEAR", cbuf);
	sprintf (cbuf, "%d %0.4f", gear_status, gear_proc);
	oapiWriteScenario_string (scn, "GEAR", cbuf);
	sprintf (cbuf, "%d %0.4f", pldoors_status, pldoors_proc);
	oapiWriteScenario_string (scn, "DOORS", cbuf);
	sprintf (cbuf, "%d %0.4f", wings_status, wings_proc);
	oapiWriteScenario_string (scn, "WINGS", cbuf);
	sprintf (cbuf, "%d %0.4f", jets_status, jets_proc);
	oapiWriteScenario_string (scn, "JETS", cbuf);
	sprintf (cbuf, "%d %0.4f", hatch_status, hatch_proc);
	oapiWriteScenario_string (scn, "HATCH", cbuf);
//	oapiWriteScenario_float (scn, "TGT_HEADING", PadHeading - 180.0);

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void SPIRAL::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		if (!strnicmp (line, "CONFIGURATION", 13)) {
		    sscanf (line+13, "%d", &stage);
		} else if (!strnicmp (line, "BAY_OPENS", 9)) {
			sscanf (line+9, "%d", &bBayOpens);
		} else if (!strnicmp (line, "SPHEREGONE", 10)) {
			sscanf (line+10, "%d", &bSphereJettisoned);
		} else if (!strnicmp (line, "CONEGONE", 8)) {
			sscanf (line+8, "%d", &bConeJettisoned);
		} else if (!strnicmp (line, "EJECTED", 7)) {
			sscanf (line+7, "%d", &bEjected);
		} else if (!strnicmp (line, "EVA", 3)) {
			sscanf (line+3, "%d", &bEVA);
		} else if (!strnicmp (line, "DEORBIT FIRED", 12)) {
			sscanf (line+12, "%d", &bDeorbitFired);
		} else if (!strnicmp (line, "THRUST TERMINATED", 17)) {
			sscanf (line+17, "%d", &bThrustTerminated);
		} else if (!strnicmp (line, "BOOSTERGEAR", 11)) {
		sscanf (line+11, "%d%lf", &bgear_status, &bgear_proc);
		} else if (!strnicmp (line, "GEAR", 4)) {
		sscanf (line+4, "%d%lf", &gear_status, &gear_proc);
		} else if (!strnicmp (line, "DOORS", 5)) {
		sscanf (line+5, "%d%lf", &pldoors_status, &pldoors_proc);
		} else if (!strnicmp (line, "WINGS", 5)) {
		sscanf (line+6, "%d%lf", &wings_status, &wings_proc);
		} else if (!strnicmp (line, "JETS", 4)) {
		sscanf (line+6, "%d%lf", &jets_status, &jets_proc);
		} else if (!strnicmp (line, "HATCH", 5)) {
		sscanf (line+5, "%d%lf", &hatch_status, &hatch_proc);
/*		} else if (!strnicmp (line, "HEADING", 7)) {
			sscanf(line+7, "%lf", &initialheading);
		} else if( !strncmp( line, "TGT_HEADING", 11 ) ) {
            sscanf (line+11, "%lf", &d_heading);
			PadHeading = d_heading + 180.0;  // It's not pad heading any more; just target heading
			INCLINATION = acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG;
			d_heading = PadHeading;
*/
		} else ParsePayloadString(line);

		ParseScenarioLineEx (line, vs);
	}



	/* always set the stage whenever something may have changed */
	SetMainStage();

	RevertBoosterLandingGear();
	RevertBoosterLandingGear();

	RevertLandingGear();
	RevertLandingGear();
	if (!bEjected) {
		RevertHatch();
		RevertHatch();
	}
	RevertWings();
	RevertWings();
	RevertPayloadBayDoors ();
	RevertPayloadBayDoors();

	ProgramAuto();

}

void SPIRAL::ProgramAuto()
{

	bAutoProgrammed = true;
	N=14;
	double *fmet = new double[N];
	double *fcpitch = new double[N];

	fmet[0]					=    0.0;
	fmet[1]					=   15.0;
	fmet[2]					=   20.0;
	fmet[3]					=   25.0;
	fmet[4]					=   35.0;
	fmet[5]					=   55.0;
	fmet[6]					=   60.0;
	fmet[7]					=  110.0;
	fmet[8]					=  255.0;
	fmet[9]					=  325.0;
	fmet[10]				=  448.0;
	fmet[11]				=  452.0;
	fmet[12]				=  457.0;
	fmet[13]				=  460.0;

	fcpitch[0]				= 45.0;
	fcpitch[1]				= 43.0;
	fcpitch[2]				= 41.0;
	fcpitch[3]				= 39.0;
	fcpitch[4]				= 37.0;
	fcpitch[5]				= 35.0;
	fcpitch[6]				= 32.0;
	fcpitch[7]				= 30.0;
	fcpitch[8]				= 20.0;
	fcpitch[9]				= 15.0;
	fcpitch[10]				= 10.0;
	fcpitch[11]				=  5.0;
	fcpitch[12]				=  2.5;
	fcpitch[13]				=  0.0;
	ProgramPitchMet( N, fmet, fcpitch );
}


void SPIRAL::PlaceComponents()
{
	COMMON_MESH tmp;

    if (stage < 1)  {  // 50-50 rocket stage (HLA)
		tmp.MeshHandle(hHLA);
		tmp.MeshPosition(HLA_OFS);
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hDummy5);
		tmp.MeshPosition(_V(0,0,0));
		AddMeshEx (tmp);
	}
	if (stage > -1) {
		if (stage < 5) {
			tmp.MeshHandle(hSPIRALBay1);
			tmp.MeshPosition(BAY1_OFS);
			AddMeshEx (tmp);
			tmp.MeshHandle(hSPIRALBay2);
			tmp.MeshPosition(BAY2_OFS);
			AddMeshEx (tmp);
			if ((!bEjected) && (!bSphereJettisoned)) {
				tmp.MeshHandle(hSeat);
				tmp.MeshPosition(EJSEAT_OFS);
				AddMeshEx (tmp);
				tmp.MeshHandle(hTopHatch);
				tmp.MeshPosition(TOPHATCH_OFS);
				AddMeshEx (tmp);
			}
			else {
				tmp.MeshHandle(hDummy1);
				tmp.MeshPosition(_V(0,0,0));
				AddMeshEx (tmp);
				tmp.MeshHandle(hDummy2);
				tmp.MeshPosition(_V(0,0,0));
				AddMeshEx (tmp);
			}

			if ((!bEVA) && (!bEjected) && (!bSphereJettisoned)) {
				tmp.MeshHandle(hAstro);
				tmp.MeshPosition(ASTRO_OFS);
				AddMeshEx (tmp);
			}
			else {
				tmp.MeshHandle(hDummy3);
				tmp.MeshPosition(_V(0,0,0));
				AddMeshEx (tmp);
			}

			tmp.MeshHandle(hSPIRAL);
			tmp.MeshPosition(SPIRAL_OFS);
			AddMeshEx (tmp);
			if (!bSphereJettisoned) {
				tmp.MeshHandle(hEjSphere);
				tmp.MeshPosition(ENTRYSHIELD_OFS);
				AddMeshEx (tmp);
			}
			else {
				tmp.MeshHandle(hDummy4);
				tmp.MeshPosition(_V(0,0,0));
				AddMeshEx (tmp);
			}
		}
		if (stage == 3) {
			tmp.MeshHandle(hSoyuzIntx);
			tmp.MeshPosition(TRANSITION_OFS);
			AddMeshEx (tmp);
		}
		if (stage < 3) {
			tmp.MeshHandle(hSpiralRB2);
			tmp.MeshPosition(TRANSITION_OFS);
			AddMeshEx (tmp);
		}
		if (stage < 2)  {
			tmp.MeshHandle(hSpiralRB);
			tmp.MeshPosition(TRANSTAGE_OFS);
			AddMeshEx (tmp);
			if (!bConeJettisoned) {
				tmp.MeshHandle(hTailcone);
				tmp.MeshPosition(TAILCONE_OFS);
				AddMeshEx (tmp);
			}
		}
	}

}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void SPIRAL::SetMainStage( )
{

	SoundOptionOnOff3(MyID,PLAYMAINTHRUST,true);

	if (stage < 1)
		SetBoosterStage();
	else if (stage == 1)
		SetSpiralRB();
	else if (stage == 2)
		SetSpiralRB2();
	else if (stage == 3)
		SetEscapeStage();
	else SetSPIRALStage();
}


void SPIRAL::SetBoosterStage( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (30);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	if (stage == -1) SetEmptyMass (HLA_DRYMASS + atchmass + CalculateMass() );
	else 	SetEmptyMass ( HLA_DRYMASS + SPIRAL_DRYMASS + RB2_DRYMASS + RB_DRYMASS + RB2_PROP_MASS + JET_PROP_MASS + atchmass + CalculateMass() );

	SetTouchdownPoints (_V(0,-7.3,14.0), _V(-9.15,-7.97,-12.5), _V(9.15,-7.97,-12.5));
	SetSurfaceFrictionCoeff (0.025, 0.4);
//	SetTouchdownPoints (_V(0,-10,-11), _V(-10,10,-11), _V(10,10,-11));
	SetPMI (_V(18,19.5,1.25));
	SetCrossSections (_V(42.4,76.4,12.5));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS_BOOSTER);
	MakeAirfoils();
//	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,1.0);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(1,MESHVIS_ALWAYS);
//	if (stage == -1) SetMeshVisibilityMode(2,MESHVIS_ALWAYS);


	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */
	if (ph_booster==NULL)  
		ph_booster  = CreatePropellantResource(HLA_PROP_MASS,HLA_PROP_MASS*InitFuelLevel,1.0); 
	if (stage > -1) {
		if (ph_first==NULL)  
			ph_first  = CreatePropellantResource(RB_PROP_MASS,RB_PROP_MASS*InitFuelLevel,1.0); 
		if (!ph_Spiral)  
			ph_Spiral  = CreatePropellantResource(RCS_PROP_MASS,RCS_PROP_MASS,1.0); 
	}

	SetDefaultPropellantResource (ph_booster); 

	th_first[0] = CreateThruster ( _V(-2.25, 1.25,-20.0), _V(0,0,1), THRUST_HLA_VAC, ph_booster, ISP_JET_VAC, ISP_JET_SL);
	AddExhaustStream (th_first[0], _V(-2.25,-2.80,-20.0), &contrail);
	th_first[1] = CreateThruster ( _V( 2.25, 1.25,-20.0), _V(0,0,1), THRUST_HLA_VAC, ph_booster, ISP_JET_VAC, ISP_JET_SL);
	AddExhaustStream (th_first[1], _V( 2.25,-2.80,-20.0), &contrail);
	th_first[2] = CreateThruster ( _V(-2.25,-1.25,-20.0), _V(0,0,1), THRUST_HLA_VAC, ph_booster, ISP_JET_VAC, ISP_JET_SL);
	AddExhaustStream (th_first[2], _V(-2.25,-5.40,-20.0), &contrail);
	th_first[3] = CreateThruster ( _V( 2.25,-1.25,-20.0), _V(0,0,1), THRUST_HLA_VAC, ph_booster, ISP_JET_VAC, ISP_HLA_SL);
	AddExhaustStream (th_first[3], _V( 2.25,-5.40,-20.0), &contrail);


	thg_first = CreateThrusterGroup (th_first, 4, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 20.0, 2.0, _V(-2.25,-2.80, -20.0), _V(0,0,-1),texLH2);
	AddExhaust (th_first[1], 20.0, 2.0, _V( 2.25,-2.80, -20.0), _V(0,0,-1), texLH2);
	AddExhaust (th_first[2], 20.0, 2.0, _V(-2.25,-5.40, -20.0), _V(0,0,-1), texLH2);
	AddExhaust (th_first[3], 20.0, 2.0, _V( 2.25,-5.40, -20.0), _V(0,0,-1), texLH2);
//	numengines = 4;

	SoundOptionOnOff3(MyID,PLAYMAINTHRUST,false);

	EnableTransponder(true);
/*
	if (stage == -3) {
		if (!ph_TitovV)  
			ph_TitovV  = CreatePropellantResource(BOOSTER_RCSMASS,BOOSTER_RCSMASS,1.0); 
		if (!ph_Jets)  
			ph_Jets  = CreatePropellantResource(BJET_PROP_MASS,BJET_PROP_MASS,1.0); 
		SetAttControlsTITOVVBoost();
		if (jets_proc > 0.0) BoosterSwitchToJets();
//	}
//	if ((GetAltitude() > 17000.0) && (stage == -3)) {
		nozzles_proc = 1.0;
		nozzles_status = DOOR_OPEN;
		RevertNozzles();		// Freshly invoked booster stage needs to start with nozzles out.  Then we retract them.
		ActivateNavmode(NAVMODE_KILLROT);
	}
*/
}

void SPIRAL::SetSpiralRB( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (20);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass ( SPIRAL_DRYMASS + RB2_DRYMASS + RB_DRYMASS + RB2_PROP_MASS +JET_PROP_MASS + atchmass + CalculateMass() );


	SetTouchdownPoints (_V(0,-1.0,-3), _V(-.7,.7,-3), _V(.7,.7,-3));
	SetPMI (_V(18,19.5,3.25));
	SetCrossSections (_V(12.67,27.48,5.40));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.05);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
	DelBoosterAnimations();
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
//	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(4,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(6,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(7,MESHVIS_ALWAYS);


	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hSPIRAL, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */
	if (ph_first==NULL)  
		ph_first  = CreatePropellantResource(RB_PROP_MASS,RB_PROP_MASS*InitFuelLevel,1.0); 
	if (!ph_Spiral)  
		ph_Spiral  = CreatePropellantResource(RCS_PROP_MASS,RCS_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_first); 

	th_first[0] = CreateThruster ( _V(0,0.0,-23.6), _V(0,0,1), THRUST_RB_VAC, ph_first, ISP_RB_VAC, ISP_RB_SL);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	thg_first = CreateThrusterGroup (th_first, 1, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 10.0, 1.25, _V(0,-0.08,-23.6), _V(0,0,-1),texLH2); 

	SetAttControlsSpiralRB();
	EnableTransponder(true);

}


void SPIRAL::SetSpiralRB2( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (14);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass ( SPIRAL_DRYMASS + RB2_DRYMASS + JET_PROP_MASS + atchmass + CalculateMass() );


	SetTouchdownPoints (_V(0,-1.0,-3), _V(-.7,.7,-3), _V(.7,.7,-3));
	SetPMI (_V(18,19.5,3.25));
	SetCrossSections (_V(12.67,27.48,5.40));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.05);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
	DelBoosterAnimations();
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
//	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(4,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(6,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(7,MESHVIS_ALWAYS);


	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hSPIRAL, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */
	if (ph_first != NULL)
		DelPropellantResource (ph_first);
	if (ph_first==NULL)  
		ph_first  = CreatePropellantResource(RB2_PROP_MASS,RB2_PROP_MASS*InitFuelLevel,1.0); 
	if (!ph_Spiral)  
		ph_Spiral  = CreatePropellantResource(RCS_PROP_MASS,RCS_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_first); 

	th_first[0] = CreateThruster ( _V(0,0.0,-6.4), _V(0,0,1), THRUST_RB2_VAC, ph_first, ISP_RB2_VAC, ISP_RB2_SL);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	thg_first = CreateThrusterGroup (th_first, 1, THGROUP_MAIN);
	
	AddExhaust (th_first[0], 4.0, 0.6, _V(0,-0.08,-6.4), _V(0,0,-1),texLH2); 

	SetAttControlsSpiralRB();
	EnableTransponder(true);

}

void SPIRAL::SetEscapeStage( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

/*	PARTICLESTREAMSPEC contrail = 
	{
		0, 5.0, 5, 150, 0.3, 4.0, 16, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};
*/
	PARTICLESTREAMSPEC exhaust_main = {
		0, 0.5, 3, 500, 0, 0.15, 50, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
			PARTICLESTREAMSPEC::ATM_FLAT, 0.3, 0.3
	};

	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (14);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass ( SPIRAL_DRYMASS + JET_PROP_MASS + ESCAPE_DRYMASS + atchmass + CalculateMass() );


	SetTouchdownPoints (_V(0,-1.0,-3), _V(-.7,.7,-3), _V(.7,.7,-3));
	SetPMI (_V(16,17.5,3.0));
	SetCrossSections (_V(12.67,27.48,5.40));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.05);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
	DelBoosterAnimations();

	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
//	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(4,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(6,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(7,MESHVIS_ALWAYS);
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_first != NULL)
		DelPropellantResource (ph_first);
	if (ph_escape1 == NULL)  
		ph_escape1  = CreatePropellantResource(ESCAPE_PROP_MASS,ESCAPE_PROP_MASS,1.0); 
	if (ph_escape2 == NULL)  
		ph_escape2  = CreatePropellantResource(ESCAPE_PROP_MASS,ESCAPE_PROP_MASS,1.0); 	
	if (ph_escape3 == NULL)  
		ph_escape3  = CreatePropellantResource(ESCAPE_PROP_MASS,ESCAPE_PROP_MASS,1.0); 	
	if (ph_Spiral==NULL)  
		ph_Spiral  = CreatePropellantResource(RCS_PROP_MASS,RCS_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_Spiral); 

	if (GetPropellantMass(ph_Spiral) == 0.0)
		SetPropellantMass(ph_Spiral,RCS_PROP_MASS);
	if (GetPropellantMass(ph_escape1) == 0.0)
		SetPropellantMass(ph_escape1,ESCAPE_PROP_MASS);
	if (GetPropellantMass(ph_escape2) == 0.0)
		SetPropellantMass(ph_escape2,ESCAPE_PROP_MASS);
	if (GetPropellantMass(ph_escape3) == 0.0)
		SetPropellantMass(ph_escape3,ESCAPE_PROP_MASS);

	th_first[0 ] = CreateThruster ( _V(-0.673, 1.457,-6.135), _V(0,-0.4,1), THRUST_ESCAPE_VAC, ph_escape1, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);
	th_first[1 ] = CreateThruster ( _V( 0.673,-1.457,-6.135), _V(0, 0.4,1), THRUST_ESCAPE_VAC, ph_escape1, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);
	th_first[2 ] = CreateThruster ( _V( 0.673, 1.457,-6.135), _V(0,-0.4,1), THRUST_ESCAPE_VAC, ph_escape1, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);
	th_first[3 ] = CreateThruster ( _V(-0.673,-1.457,-6.135), _V(0, 0.4,1), THRUST_ESCAPE_VAC, ph_escape1, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);
	th_first[4 ] = CreateThruster ( _V(-1.100, 1.200,-6.135), _V(-0.1,-0.37,1), THRUST_ESCAPE_VAC, ph_escape2, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);
	th_first[5 ] = CreateThruster ( _V( 1.100,-1.200,-6.135), _V( 0.1, 0.37,1), THRUST_ESCAPE_VAC, ph_escape2, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);
	th_first[6 ] = CreateThruster ( _V( 1.100, 1.200,-6.135), _V( 0.1,-0.37,1), THRUST_ESCAPE_VAC, ph_escape2, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);
	th_first[7 ] = CreateThruster ( _V(-1.100,-1.200,-6.135), _V(-0.1, 0.37,1), THRUST_ESCAPE_VAC, ph_escape2, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);
	th_first[8 ] = CreateThruster ( _V(-1.480, 0.710,-6.135), _V(-0.2,-0.3,1), THRUST_ESCAPE_VAC, ph_escape3, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);
	th_first[9 ] = CreateThruster ( _V( 1.480,-0.710,-6.135), _V( 0.2, 0.3,1), THRUST_ESCAPE_VAC, ph_escape3, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);
	th_first[10] = CreateThruster ( _V( 1.480, 0.710,-6.135), _V( 0.2,-0.3,1), THRUST_ESCAPE_VAC, ph_escape3, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);
	th_first[11] = CreateThruster ( _V(-1.480,-0.710,-6.135), _V(-0.2, 0.3,1), THRUST_ESCAPE_VAC, ph_escape3, ISP_ESCAPE_VAC, ISP_ESCAPE_SL);

	AddExhaustStream (th_first[0 ], _V(-0.673, 1.457,-6.135), &exhaust_main);
	AddExhaustStream (th_first[1 ], _V( 0.673,-1.457,-6.135), &exhaust_main);
	AddExhaustStream (th_first[2 ], _V( 0.673, 1.457,-6.135), &exhaust_main);
	AddExhaustStream (th_first[3 ], _V(-0.673,-1.457,-6.135), &exhaust_main);
	AddExhaustStream (th_first[4 ], _V(-1.100, 1.200,-6.135), &exhaust_main);
	AddExhaustStream (th_first[5 ], _V( 1.100,-1.200,-6.135), &exhaust_main);
	AddExhaustStream (th_first[6 ], _V( 1.100, 1.200,-6.135), &exhaust_main);
	AddExhaustStream (th_first[7 ], _V(-1.100,-1.200,-6.135), &exhaust_main);
	AddExhaustStream (th_first[8 ], _V(-1.480, 0.710,-6.135), &exhaust_main);
	AddExhaustStream (th_first[9 ], _V( 1.480,-0.710,-6.135), &exhaust_main);
	AddExhaustStream (th_first[10], _V( 1.480, 0.710,-6.135), &exhaust_main);
	AddExhaustStream (th_first[11], _V(-1.480,-0.710,-6.135), &exhaust_main);

	th_fake[0] = CreateThruster ( _V(0.0,0.0,-6.135), _V(0,0,1), 0.0001, ph_escape3, ISP_ESCAPE_VAC*100, ISP_ESCAPE_SL*100);


	thg_first2 = CreateThrusterGroup (th_first, 12, THGROUP_USER);
	thg_first = CreateThrusterGroup (th_fake, 1, THGROUP_MAIN);

/*
	Exh1 = AddExhaust (th_first[0], 2.0, 0.3, tex);
	Exh2 = AddExhaust (th_first[1], 2.0, 0.3, tex);
	Exh3 = AddExhaust (th_first[2], 2.0, 0.3, tex);
	Exh4 = AddExhaust (th_first[3], 2.0, 0.3, tex);
*/
	SetAttControlsSpiralEscape();
	EnableTransponder(true);
}

void SPIRAL::SetSPIRALStage( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (10);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass ( SPIRAL_DRYMASS + atchmass + CalculateMass() );


	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);
	retex = oapiRegisterReentryTexture("reentry");
	SetReentryTexture(retex,6e6,1,1);


	SetTouchdownPoints (_V(0,-1.0,-3), _V(-.7,.7,-3), _V(.7,.7,-3));
	SetPMI (_V(5.46,6.16,1.22));
	SetCrossSections (_V(11.67,23.48,4.40));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.05);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
	DelBoosterAnimations();

	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
//	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(4,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(6,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(7,MESHVIS_ALWAYS);


	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_booster != NULL)
		DelPropellantResource (ph_booster);
	if (ph_first != NULL)
		DelPropellantResource (ph_first);
	if (ph_escape3 != NULL)	{	
		DelPropellantResource (ph_escape1); 
		DelPropellantResource (ph_escape2); 
		DelPropellantResource (ph_escape3); 
	}

	if (ph_Spiral==NULL)  
		ph_Spiral  = CreatePropellantResource(RCS_PROP_MASS,RCS_PROP_MASS,1.0); 
	if (ph_Jets==NULL)  
		ph_Jets  = CreatePropellantResource(JET_PROP_MASS,JET_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_Spiral); 

	if (GetPropellantMass(ph_Spiral) == 0.0)
		SetPropellantMass(ph_Spiral,RCS_PROP_MASS);
	if (GetPropellantMass(ph_Jets) == 0.0)
		SetPropellantMass(ph_Jets,JET_PROP_MASS);

	th_first[0] = CreateThruster ( _V(0,0,-4.1), _V(0,0,1), THRUST_SPIRAL_VAC, ph_Spiral, ISP_SPIRAL_VAC, ISP_SPIRAL_SL);

	thg_first = CreateThrusterGroup (th_first, 1, THGROUP_MAIN);
	AddExhaust (th_first[0], 3.0, 0.3, _V(0,-0.083,-4.1), _V(0,-0.02,-1),tex);

	SetAttControlsSPIRAL();
	EnableTransponder(true);

	if (jets_status==DOOR_OPEN)
		SwitchToJets();
/*	if ((GetMachNumber() < 5.15) && (GetAtmPressure() > 100.0) && (wings_status==DOOR_OPEN)) {
		wings_status=DOOR_CLOSED;
		wings_proc=0.0;
		RevertWings();
		RevertWings();
	}
*/
}


void SPIRAL::JettisonPart()
 {
				VESSELSTATUS vs;
				VECTOR3 ofs = _V(0,0,0);
				VECTOR3 vel = _V(0,0,-0.4);
				if (stage == 0)	
					vel = _V(0,-8.0,0.0);
				GetStatus (vs);
				vs.eng_main = vs.eng_hovr = 0.0;
				SetEngineLevel(ENGINE_MAIN, 0);
				VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				vs.vrot.x = 0.0;
				if (stage == 0)
					vs.vrot.x = -0.125;
				vs.vrot.y = 0.0;
				vs.vrot.z = 0.0;
				char name[256];
				if (stage == 0) {
					DelBoosterAnimations();
					strcpy (name, GetName()); strcat (name, "-Booster");
					OBJHANDLE BSTR = oapiCreateVessel (name, "SPIRAL/HLA", vs);
					VESSEL *remotevessel = oapiGetVesselInterface(BSTR);
					remotevessel->SetFuelMass(GetPropellantMass(ph_booster));
					DelPropellantResource (ph_booster); 
					PlayVesselWave3(MyID,PLAYBANG);
				}
				else if (stage == 1) {
					strcpy (name, GetName()); strcat (name, "-SpiralRB");
					VESSEL::Create (name, "SPIRAL/SpiralRB", vs);	
					DelPropellantResource (ph_first); 
					PlayVesselWave3(MyID,PLAYBANG);
				}
				else if (stage == 2) {
					strcpy (name, GetName()); strcat (name, "-SpiralRB Stage 2");
					VESSEL::Create (name, "SPIRAL/SpiralRB2", vs);	
					DelPropellantResource (ph_first); 
					PlayVesselWave3(MyID,PLAYBANG);
					stage++;			// Skip the escape stage step
				}
				else if (stage == 3) {
					strcpy (name, GetName()); strcat (name, "-Escape Stage");
					VESSEL::Create (name, "SPIRAL/SoyuzIntx", vs);
					DelPropellantResource (ph_escape1); 
					DelPropellantResource (ph_escape2); 
					DelPropellantResource (ph_escape3); 
					PlayVesselWave3(MyID,PLAYBANG);
				}
				else stage--;


				stage++;

 }

void SPIRAL::AttachSweep()
{

	
			/* Now scan for nearby attach points; these need to stay in the bay!  */
				
			Local2Global (_V(0,0,0.0), grms);  // global position of RMS tip
			
			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (i = 0; i < oapiGetVesselCount(); i++) {
				hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < oapiGetSize (hV)) { // in range
					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
						if (strncmp (id, "BOLT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						if (dist (gpos, grms) < 0.25) { // found one!
							// check whether satellite is currently clamped into payload bay
							if ((hV == GetAttachmentStatus (Baypos)) && (!Firstsweep))
							{
								AttachChild (hV, Baypos, hAtt);
								atchmass = v->GetMass();
								BayVessel = hV;
								BayV = oapiGetVesselInterface (BayVessel);
							}
							else
							{	
								AttachChild (hV, Baypos, hAtt);
								atchmass = v->GetMass();
								hDock = v->GetDockHandle(0);
								BayVessel = hV;
								BayV = oapiGetVesselInterface (BayVessel);
/*								if (hDock != NULL) {
									v->GetDockParams(hDock,pos, dir, rot);
									pos.z += 4.8;
//									CreateDock(pos, dir, rot);
									SetDockParams(Dock0, pos, dir, rot);
									v->DelDock(hDock);
								}
								else {
									Undock(ALLDOCKS);
									DelDock(Dock0);
								}
*/
							}
//							return 1;
						}
					}
				}
			}
}

int SPIRAL::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	if (!fPressed) return 0; // only process keydown events
	if (KEYMOD_SHIFT (keystate)) 
	{
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
		if (KEYDOWN (keystate, OAPI_KEY_E))
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				// Report the inclination setpoint
				// Eject Eject Eject!!!!

			if (stage < 1) return 1;
			if ((bSphereJettisoned) || (bEjected)) return 1;
			if ((GetMachNumber() > 1.0) || (GetAtmPressure() < 10.0)) 
					JettisonSphere();

			else if (!bEjected) {

				PlayVesselWave3(MyID,PLAYBANG);
				bEjected = true;
				VESSELSTATUS vs;
				GetStatus (vs);
				vs.eng_main = vs.eng_hovr = 0.0;
				vs.fuel=1.0;
				vs.status=0;
				VECTOR3 ofs = TOPHATCH_OFS;
				VECTOR3 vel = _V(0,20.0,-5.0);
				VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				vs.vrot.x = 0.7;
				vs.vrot.y = 0.0;
				vs.vrot.z = 0.0;
				char name[256];
				strcpy (name, GetName()); strcat (name, "-Hatch");
				VESSEL::Create (name, "SPIRAL/SPIRAL_TopHatch", vs);

				GetStatus (vs);
				ofs = ASTRO_OFS;
				vel = _V(0,12.0,0.0);
	//			rofs = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	//			rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				vs.vrot.x = 0.0;
				vs.vrot.y = 0.0;
				vs.vrot.z = 0.0;
	//			char name[256];
				strcpy (name, GetName()); strcat (name, "-Pilot");
				VESSEL::Create (name, "SPIRAL/SpiralPLT", vs);
				OBJHANDLE hvessel;
				hvessel=oapiGetVesselByName(name);
				if (hvessel != 0) 
					oapiSetFocusObject(hvessel);
				SetMainStage();
			}
																
			return 1;
		}
		return 0;
	}
	else { // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0)) {
				if (stage == 3)
					if ((GetThrusterLevel(th_first[0]) > 0.0) || (GetThrusterLevel(th_first[4]) > 0.0) 
						|| (GetThrusterLevel(th_first[8]) > 0.0)) return 0;

					if ((stage < 4) && (stage > -1) && (pldoors_proc == 0.0)) {
						JettisonPart();
					}
					else {
						OverridePayloadVelocity(_V(0,+0.25,0));
						DeployPayload();
					}

					/* CVE-lite function ! */
					/* IMPORTANT */
					/* call DeployPayload() to jettison the GPDS payloads in sequence */

					AttachSweep();

					/* IMPORTANT */
					/* Always re-set the stage after any kind of change */
					SetMainStage();
					if (stage == 1) SetThrusterGroupLevel (THGROUP_MAIN,1.0);
;
				
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_G)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_G, 1.0)) {
				// Toggle the Gear
				if (stage < 1)
					RevertBoosterLandingGear();
				else
					RevertLandingGear ();
				StopVesselWave3(MyID,PLAYGEAR);
				PlayVesselWave3(MyID,PLAYGEAR);
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_K)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_K, 1.0)) {
				if (stage < 1) return 1;
				// Toggle the bay doors
				if (bBayOpens)
					RevertPayloadBayDoors ();
				if (BayVessel != NULL)
					SendCentralMessage(BayV, "MSG", "REVERTPROTRUSIONS");
				if (pldoors_status == DOOR_CLOSING)
					Undock(ALLDOCKS);
			}
			return 1;
		}
/*		if (KEYDOWN (keystate, OAPI_KEY_B)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_B, 1.0)) {
				// Toggle the wings
				if (BrakesOn)
					BrakesOn = false;
				else BrakesOn = true;
				RevertBrakes();
			}
			return 1;
		}
*/
		if (KEYDOWN (keystate, OAPI_KEY_P)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_P, 1.0)) {
				// Toggle the top hatch
				if (stage < 1) return 1;
				if (!bEjected)
					RevertHatch ();
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_O))
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_O, 1.0))
				if (stage > 2) {
				AutoPilot = false;
					ActivateNavmode(NAVMODE_KILLROT);
					SetThrusterGroupLevel (thg_first,0.0);	
				}
				else if (!guidance) { 
					Guidance( true );
//					guidance=true;
					sprintf(oapiDebugString(),"Guidance ON");
					pitchinput = 0.0;
					yawinput = 0.0;
					rollinput = 0.0;
					if (stage == 0) {
						JettisonPart();
						SetMainStage();
						SetThrusterGroupLevel (THGROUP_MAIN,1.0);
					}
				}
				else {
					Guidance( false );
//					guidance = false;
					SetAttitudeRotLevel(_V(0,0,0));
					pitchinput = 0.0;
					yawinput = 0.0;
					rollinput = 0.0;

					SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
					SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
					sprintf(oapiDebugString(),"Guidance OFF");
				}
			return 1;
		}
/*		if (KEYDOWN (keystate, OAPI_KEY_U)) 
		{ // "Set launch Azimuth"
			if (oapiAcceptDelayedKey (OAPI_KEY_U, 1.0))
			{
				GetInclinationSP();
				return 1;
			}
		}
*/		if (KEYDOWN (keystate, OAPI_KEY_E)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0)) {
				// EVA
				if ((hatch_status == DOOR_OPEN) && (!bEjected))
				{
					char name[256];
					OBJHANDLE hvessel;
					strcpy (name, GetName()); strcat (name, "-PLT");
					hvessel=oapiGetVesselByName(name);
					if (hvessel == 0)
					{
						bEVA = true;
						SeparateEVA();
						SetMainStage();
						hvessel=oapiGetVesselByName(name);
						if (hvessel != 0)
							oapiSetFocusObject(hvessel);
					}
					else
					{
						oapiDeleteVessel(hvessel);
						bEVA = false;
						SetMainStage();
					};
				}

			}
			return 1;
		}

	}
	return 0;
}



int SPIRAL::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}




void SPIRAL::SetAttControlsSpiralRB()
{
	THRUSTER_HANDLE th_rot[2];
	th_rot[0] = CreateThruster (_V(0,0, 10), _V(0, 1,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-10), _V(0,-1,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,0, 10), _V(0,-1,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-10), _V(0, 1,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 10), _V(-1,0,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-10), _V( 1,0,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 10), _V( 1,0,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-10), _V(-1,0,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V( 6,0,0), _V(0,-1,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(-6,0,0), _V(0, 1,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V( 6,0, 0), _V(0, 1,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(-6,0,0), _V(0, -1,0), MAX_ATT_FIRST*5, ph_first, ISP_SPIRAL_VAC);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);
}

void SPIRAL::SetAttControlsSpiralEscape()
{

	THRUSTER_HANDLE th_rot[2];
	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0, 1,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0,-1,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0,0), _V(0,-1,0), MAX_ATT_FIRST/2, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(-2,0,0), _V(0, 1,0), MAX_ATT_FIRST/2, ph_Spiral, ISP_SPIRAL_VAC);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(2,0, 0), _V(0, 1,0), MAX_ATT_FIRST/2, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(-2,0,0), _V(0, -1,0), MAX_ATT_FIRST/2, ph_Spiral, ISP_SPIRAL_VAC);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

}


void SPIRAL::SetAttControlsSPIRAL()
{
	THRUSTER_HANDLE th_rot[2];
	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0, 1,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0,-1,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.6, 0.07,-4.25), _V(0, 1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.6, 0.07,-4.25), _V(0, 1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.633, 0.288, 3.47), _V(-0.7,-0.7,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.633, 0.288, 3.47), _V( 0.7,-0.7,0.0),texRCS);  // Spiral
	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_FIRST/4, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_FIRST/4, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 0.5, 0.15, _V(-1.6,-0.105,-4.25), _V(0,-1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.5, 0.15, _V( 1.6,-0.105,-4.25), _V(0,-1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.633, 0.288, 3.47), _V(-0.7,-0.7,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.633, 0.288, 3.47), _V( 0.7,-0.7,0.0),texRCS);  // Spiral
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.6, 0.07,-4.25), _V(0, 1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.6, 0.07,-4.25), _V(0, 1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.548,0.35, 3.47), _V(0, 1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.548,0.35, 3.47), _V(0, 1.0,0.0),texRCS);  // Spiral
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.6,-0.105,-4.25), _V(0,-1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.6,-0.105,-4.25), _V(0,-1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.548,0.35, 3.47), _V(0, 1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.548,0.35, 3.47), _V(0, 1.0,0.0),texRCS);  // Spiral
	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.71,-0.02,-4.25), _V(-1.0,0,0.0),texRCS);  // SpiralRB
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.633, 0.288, 3.47), _V( 1.0,0,0.0),texRCS);  // Spiral
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.71,-0.02,-4.25), _V( 1.0,0,0.0),texRCS);  // SpiralRB
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.633, 0.288, 3.47), _V(-1.0,0,0.0),texRCS);  // Spiral
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.71,-0.02,-4.25), _V( 1.0,0,0.0),texRCS);  // SpiralRB
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.633, 0.288, 3.47), _V( 1.0,0,0.0),texRCS);  // Spiral
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(1,0,0),MAX_ATT_FIRST, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.71,-0.02,-4.25), _V(-1.0,0,0.0),texRCS);  // SpiralRB
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.633, 0.288, 3.47), _V(-1.0,0,0.0),texRCS);  // Spiral
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V(2,0,0), _V(0,-1,0), MAX_ATT_FIRST/2, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(-2,0,0), _V(0, 1,0), MAX_ATT_FIRST/2, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.6, 0.07,-4.25), _V(0, 1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.6,-0.105,-4.25), _V(0,-1.0,0.0),texRCS);  // Spiral
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(2,0, 0), _V(0, 1,0), MAX_ATT_FIRST/2, ph_Spiral, ISP_SPIRAL_VAC);
	th_rot[1] = CreateThruster (_V(-2,0,0), _V(0, -1,0), MAX_ATT_FIRST/2, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-1.6, 0.07,-4.25), _V(0, 1.0,0.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 1.6,-0.105,-4.25), _V(0,-1.0,0.0),texRCS);  // Spiral
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_FIRST*2, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 1.2, 0.35, _V(-0.2,0.02,-4.2), _V(0, 0,-1.0),texRCS);  // Spiral
	AddExhaust (th_rot[0], 1.2, 0.35, _V( 0.2,0.02,-4.2), _V(0, 0,-1.0),texRCS);  // Spiral
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_FIRST/2, ph_Spiral, ISP_SPIRAL_VAC);
	AddExhaust (th_rot[0], 0.8, 0.25, _V(-0.633, 0.288, 3.47), _V(-0.7,0,0.7),texRCS);  // Spiral
	AddExhaust (th_rot[0], 0.8, 0.25, _V( 0.633, 0.288, 3.47), _V( 0.7,0,0.7),texRCS);  // Spiral
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
}