/**
*
* $Id: SPIRAL.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/cvelmessage.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include "../../include/guidance.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

// All data from buran.ru
const double ISP_RB_SL			=     436.0*G; 
const double ISP_RB_VAC			=     451.0*G; //***
const double ISP_RB2_SL			=     440.0*G; 
const double ISP_RB2_VAC		=     453.0*G; //***
const double ISP_ESCAPE_SL		=     221.0*G; 
const double ISP_ESCAPE_VAC		=     245.0*G; //***
const double ISP_SPIRAL_SL		=     335.0*G; 
const double ISP_SPIRAL_VAC		=     335.0*G; //***
const double ISP_JET_SL			=    8500.0*G;
const double ISP_JET_VAC		=   10500.0*G;
const double ISP_HLA_SL			=     400.0*G;
const double ISP_HLA_VAC		=     442.0*G;

const double THRUST_RB_VAC		=   68000.0*G; //***
const double THRUST_RB2_VAC		=    6595.0*G; //***
const double THRUST_ESCAPE_VAC	=	 4140.0*G; //***
const double THRUST_SPIRAL_VAC	=    1500.0*G; //
const double THRUST_JET_VAC		=    3350.0*G; //
const double THRUST_HLA_VAC		=   33000.0*G; //
const double MAX_ATT_FIRST		=     250.0*G;
const double RB_PROP_MASS		=   38250.0; //*** the total mass of fueled SpiralRB is 45950kg
const double RB2_PROP_MASS		=    4250.0; //*** the total mass of fueled SpiralRB2 is 5170kg
const double HLA_PROP_MASS		=   18000.0; //***
const double ESCAPE_PROP_MASS	=	 1200.0;
const double RCS_PROP_MASS		=	 2500.0;
const double JET_PROP_MASS		=	  500.0;
const double SPIRAL_DRYMASS		=    4190.0; //***
const double ESCAPE_DRYMASS		=    1050.0; //***
const double RB_DRYMASS			=    7500.0; //***
const double RB2_DRYMASS		=     920.0; //***
const double HLA_DRYMASS		=   36000.0; //***

const double R_EARTH = 6371000;			// radius of the earth (m)
const double U_EARTH = 3.986e5 * 1000 * 1000 * 1000; //( m^3/sec^2)
const double APO_ALT = 240.0;


const VECTOR3 SPIRAL_OFS		= _V(0,0,0);
const VECTOR3 HLA_OFS			= _V(0,0,0);
const VECTOR3 BAY1_OFS			= _V(0,0,0);
const VECTOR3 BAY2_OFS			= _V(0,0,0);
const VECTOR3 TOPHATCH_OFS		= _V(0,0,0);
const VECTOR3 ENTRYSHIELD_OFS	= _V(0,0,0);
const VECTOR3 EJSEAT_OFS		= _V(0.0, 0.10,1.4);
const VECTOR3 ASTRO_OFS			= _V(0.0, 0.25,1.6);
const VECTOR3 TRANSITION_OFS	= _V(0,0,0);
const VECTOR3 TRANSTAGE_OFS		= _V(0,0,0);
const VECTOR3 TAILCONE_OFS		= _V(0,0,0);
const VECTOR3 CAMERA_OFS		= _V( 0.0, 0.9,1.68);
const VECTOR3 CAMERA_OFS_BOOSTER= _V(-0.55,-2.57,12.9);

const double GEAR_OPERATING_SPEED = 0.25;
const double PLDOORS_OPERATING_SPEED = 0.07125;
const double WINGS_OPERATING_SPEED = 0.25;
const double JETS_OPERATING_SPEED = 0.5;
const double HATCH_OPERATING_SPEED = 0.125;

const double ROLL = 0.0;            // 180.0 to go to orbit heads-down.  Intermediate values probably won't work.

#define PLAYGEAR					 20		// the gear sound
#define PLAYBANG					 24		// the jettison sound
#define PLAYJETT					 25     // entry shield jettison sound
#define PLAYJET						 26     // the jet noise sound
#define PLAYROCK					 27     // the rocket noise sound

/* make your own vessel class (named SPIRAL here, for example)
   subclassed from COMMON_VESSEL_EX */

class SPIRAL : public COMMON_VESSEL_EX, GUIDANCE {
public:
	SPIRAL (OBJHANDLE hObj, int fmodel);
	~SPIRAL();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } gear_status, bgear_status, pldoors_status, brakes_status, hatch_status, wings_status, jets_status, nozzles_status;

	void MakeAnimations();
	void MakeBoosterAnimations();
	void DelBoosterAnimations();
	void MakeAirfoils();
	void MakeControlSurfaces();
	void UpdateAnimations();
	void ActivateLandingGear (DoorStatus action);
	void RevertLandingGear();
	void ActivateBoosterLandingGear (DoorStatus action);
	void RevertBoosterLandingGear();
	void ActivatePayloadBayDoors (DoorStatus action);
	void RevertPayloadBayDoors();
	void ActivateWings (DoorStatus action);
	void RevertWings();
	void ActivateJets (DoorStatus action);
	void RevertJets();
	void ActivateHatch (DoorStatus action);
	void RevertHatch();
	void SwitchToJets();
	void SeparateEVA();
	void SetGearParameters (double state);
	void AirBreathingEngine();
	void AirBreathingEngine2();
	void JettisonSphere();
	void JettisonCone();
	void AttachSweep();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VectorThrust();
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void ProgramAuto();
	void PlaceComponents();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage();  
	void SetBoosterStage();
	void SetSpiralRB();
	void SetSpiralRB2();
	void SetEscapeStage();
	void SetSPIRALStage();

	void SetAttControlsSpiralRB();
	void SetAttControlsSpiralEscape();
	void SetAttControlsSPIRAL();    // this is to set up the thrusters. I like to keep them separate.
	void JettisonPart();
	bool AzInput (void *id, char *str, void *data);
	bool IncInput (char *rstr);
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);
	double pitchinput, yawinput, rollinput;
	VESSELSTATUS vst;
	int stage;

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);


protected:
	double GetCPitch(double t);
	void Lateral_Guidance(VESSEL *vessel, double SimT, double SimT_Last);
	void Guidance_Routine(VESSEL *vessel, double SimT, double SimT_Last);
	void GetInclinationSP();
	void InitialiseGuidance();


	bool Guidance() { return guidance; };
	void Guidance( bool g ) { guidance = g; };

	void DisplayInclinationSetPoint( bool d ) { DisplayIncSP=d; };
	bool DisplayInclinationSetPoint() { return DisplayIncSP; };

	double initialgamma, initialheading, launchrotate;


	PROPELLANT_HANDLE ph_booster,ph_first,ph_Spiral,ph_escape1,ph_escape2,ph_escape3,ph_Jets;
	THRUSTER_HANDLE   th_first[12], th_fake[1];
	THGROUP_HANDLE    thg_first,thg_first2,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hSPIRAL,hHLA,hSPIRALBay1,hSPIRALBay2,hTopHatch,hEjSphere,hSeat,hAstro,hSoyuzIntx,hSpiralRB,hSpiralRB2,hTailcone,hDummy1,hDummy2,hDummy3,hDummy4,hDummy5;
	ANIMATIONCOMPONENT_HANDLE wing11, wing12, wing13, wing14, Gear11, Gear12, Gear13, Gear14, Gear15, Gear16, Gear17,
		Gear18, Gear19, Gear1A, Elev11, Rudd11, Elev12, Rudd12, Elev13, Rudd13, Elev14, Rudd14;
	SURFHANDLE		  tex,texLH2,texRCS, retex;
	UINT anim_rudder,anim_laileron,anim_raileron,anim_lelevator,anim_relevator,anim_lelevatortrim,anim_relevatortrim;
	UINT anim_gear;     // handle for landing gear animation
	UINT anim_bgear;     // handle for landing gear animation
	UINT anim_pldoors;     // handle for payload door animation	
	UINT anim_jets;		// handle for jets animation
	UINT anim_wings;	// handle for wings animation
	UINT anim_hatch;	// handle for top hatch animation
	double gear_proc;
	double bgear_proc;
	double pldoors_proc;
	double wings_proc;
	double jets_proc;
	double hatch_proc;
	bool BrakesOn, bBayOpens, bSphereJettisoned, bDeorbitFired, bThrustTerminated, bEjected, bEVA, bAbort, AbortChecked, bConeJettisoned;
	UINT Exh1,Exh2,Exh3,Exh4;
	PSTREAM_HANDLE stream_a_1,stream_b_1,stream_a_2,stream_b_2,stream_a_3,stream_b_3,stream_a_4,stream_b_4;
	VECTOR3 Vold, Vnew;
	double dt, oldsimt;
	int MyID;
	ATTACHMENTHANDLE Baypos;
	VECTOR3 gpos, grms, pos, dir, rot;
	OBJHANDLE hV, BayVessel;
	VESSEL *BayV;
	DWORD i, j;
	double atchmass;
	double a,e,h,V_periapsis,Altitude;
	DOCKHANDLE hDock, Dock0;
	bool Firstsweep, AutoPilot;
	char *key;
	char *message;
	VECTOR3 Rel_Vel;
	double Velocity, MachPitch;
	ANIMATIONCOMPONENT_HANDLE wing1, wing2, wing3, wing4;
	double tempP, tempY, tempR;
	VECTOR3 AngularVel;
	bool autopilot;
	bool engineignited, bgoingsouth;
	double last_simt;
	double ignition_time;
	double density;
	double displaytime, simt, d_heading;
	bool first_guidance_pass, bBoosterMade, bControlsMade, bAutoProgrammed;
	bool Burn1, Burn2, Burn3;
	double MET, diff, target, hdg, diverge, yawangle, targetrate, rate, amount;
	AIRFOILHANDLE BstrWing, BstrRudder;
	int erasecount;

};






/* basic vessel class setup */

void SPIRAL::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void SPIRAL::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void SPIRAL::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void SPIRAL::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void SPIRAL::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void SPIRAL::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int SPIRAL::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}

bool SPIRAL::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	// show gear deployment status
	if (stage == 0) {
		if (bgear_status == DOOR_OPEN || (bgear_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
			int d = hps->Markersize/2;
			int cx = hps->CX, cy = hps->CY;
			if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
				skp->Rectangle (cx-d/2, cy-d*5, cx+d/2, cy-d*4);
				skp->Rectangle (cx-d*3, cy-d*2, cx-d*2, cy-d);
				skp->Rectangle (cx+d*2, cy-d*2, cx+d*3, cy-d);
			}
		}
	} else if (stage > 2) {
		if (gear_status == DOOR_OPEN || (gear_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
			int d = hps->Markersize/2;
			int cx = hps->CX, cy = hps->CY;
			if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
				skp->Rectangle (cx-d*3, cy-d*5, cx-d*2, cy-d*4);
				skp->Rectangle (cx+d*2, cy-d*5, cx+d*3, cy-d*4);
				skp->Rectangle (cx-d*3, cy-d*2, cx-d*2, cy-d);
				skp->Rectangle (cx+d*2, cy-d*2, cx+d*3, cy-d);
			}
		}
	}


	// show pldoors deployment status
	if (stage > 2)
	if (pldoors_status == DOOR_OPEN || (pldoors_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
//			skp->Arc (cx, cy-d*8.5, cx+d*4, cy-d*6.5, cx, cy-d*6, cx+d*4, cy-d*7.5);
//			skp->Arc (cx-d*4, cy-d*8.5, cx, cy-d*6.5, cx-d*4, cy-d*7.5, cx, cy-d*6);
			skp->Line(cx-d*4, cy-d*8.5, cx-d*3, cy-d*7.5);
			skp->LineTo(cx-d*2, cy-d*7.0);
			skp->LineTo(cx-d, cy-d*6.65);
			skp->LineTo(cx, cy-d*6.5);
			skp->LineTo(cx+d, cy-d*6.65);
			skp->LineTo(cx+d*2, cy-d*7.0);
			skp->LineTo(cx+d*3, cy-d*7.5);
			skp->LineTo(cx+d*4, cy-d*8.5);
			skp->Text (cx-d, cy-d*8, "BAY", 3);
		}
	}


	// show hatch deployment status
	if (stage > 2)
	if (hatch_status == DOOR_OPEN || (hatch_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			skp->Text (cx+d*5, cy+d*1.2, "DOOR", 4);
		}
	}

	//Draw Equivalent airspeed & pressure altitude
	if(GetAtmPressure() > 0.0)
	{
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * cx;
		const double EAS_EXP = 2.0/7.0;
		const double P0 = 1.01325E6;

		double fEAS = 661.48 * sqrt(5.0 * GetAtmPressure()/ATMP * (pow(GetDynPressure() / GetAtmPressure() + 1, EAS_EXP) - 1.0));

		sprintf(pszBuffer, "%6.1f KEAS", fEAS);

		skp->Text(bx, cy - d, pszBuffer, 11);
	}
	if (GetAtmPressure() > 7000.0)				  // About 60,000'; use pressure altitude
	{
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * cx;
		const double EAS_EXP = 2.0/7.0;
		const double P0 = 1.01325E6;

		bx = 0.5 * (cx + hps->W) - d * 8;

		double fHp = (1.0 - pow(GetAtmPressure()/ATMP, 0.190263)) * (288.15/0.00198122);

		long H_press = (long)(fHp);
		H_press = H_press - H_press % 50;

		sprintf(pszBuffer, "%6d'  ", H_press);

		skp->Text(bx, cy - d, pszBuffer, 9);
	} else {									// Do inertial altitude instead
		int d = hps->Markersize/2;		
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * cx;
		const double EAS_EXP = 2.0/7.0;
		const double P0 = 1.01325E6;

		bx = 0.5 * (cx + hps->W) - d * 8;

		double fIalt = GetAltitude()/0.3048;

		long I_Alt = (long)(fIalt);
		I_Alt = I_Alt - I_Alt % 50;

		sprintf(pszBuffer, "%6d' I", I_Alt);

		skp->Text(bx, cy - d, pszBuffer, 9);
	}

	//Draw Radar altitude
	if(GetAltitude() < 2500.0 * 0.3048)
	{
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * (cx + hps->W) - d * 8;
		const double GEAR_Y = -1.35;

		sprintf(pszBuffer, "%6dR", static_cast<long>((GetAltitude() + GEAR_Y)/0.3048));

		skp->Text(bx, cy + d, pszBuffer, 7);
	}


	//Draw G-Meter

		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;

		char pszBuffer[40];
		int bx = 0.5 * cx;

		VECTOR3 ForceVec, WeightVec;
		GetForceVector(ForceVec);
		GetWeightVector(WeightVec);
		ForceVec -= WeightVec;
		double G_level = sqrt(pow(ForceVec.x,2)+pow(ForceVec.y,2)+pow(ForceVec.z,2)) / (9.81*GetMass());

		sprintf(pszBuffer, "  %.2f G", G_level);

		skp->Text(bx, cy + d, pszBuffer, strlen(pszBuffer));



	return true;
}

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new SPIRAL (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (SPIRAL*)vessel;
}

