/**
*
* $Id: SPIRAL.cpp
* (c) 2005 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "SPIRALSPHERE.h"


// 1. vertical lift component (wings and body)
// Borrowed from Mike Norman's Shuttle code

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	//                              -180 -165  -150 -135  -120 -105 -90 -75.0  -60.0  -45.0  -30.0  -15.0   0.0   15.0      30.0     45.0  60.0  75.0  90.0 105.0  120.0  135.0  150.0  165.0 180.0  
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38,  -0.5,  -0.5,  0.02, 0.655,     0.46,   0.32, 0.12, 0.05, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,     0,  0.002,  0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	// change for drag increase at high Mach:
	// Subsonic Equivalents

	double Cl0, Cl1, Cd0, Cd1, Cd2;
	if(M>=5.0) {
		Cl0 = -0.187040;
		Cl1 =  1.275600;
		Cd0 =  0.178540;
		Cd1 = -0.402900;
		Cd2 =  1.539960;
		*cl = Cl0 + Cl1*aoa;
		*cd = Cd0 + Cd1*aoa + Cd2*aoa* aoa;
	}
 	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cm = (CM[idx] + (CM[idx+1]-CM[idx])*d);
	if(M<1.25) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.04 + 0.4*oapiGetInducedDrag (*cl, 2.1, 0.82);
	}
	else if(M<5.0) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.07 + oapiGetInducedDrag (*cl, 2.1, 0.65);
// 		*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
	}
}

// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	
	if (M > 1.0) *cd *= 2.2;
	else *cd *= 0.6;
}


SPIRAL::SPIRAL (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hTopHatch			= oapiLoadMeshGlobal ("SPIRAL/TopHatch");	
	hEjSphere			= oapiLoadMeshGlobal ("SPIRAL/EjSphere");
	hSeat				= oapiLoadMeshGlobal ("SPIRAL/ejseat");
	hAstro				= oapiLoadMeshGlobal ("SPIRAL/USSR-eva");
	hDummy1				= oapiLoadMeshGlobal ("SPIRAL/dummy1");
	hDummy2				= oapiLoadMeshGlobal ("SPIRAL/dummy2");
	hDummy3				= oapiLoadMeshGlobal ("SPIRAL/dummy3");
	hDummy4				= oapiLoadMeshGlobal ("SPIRAL/dummy4");

	/* set the propellant handles to NULL so I can check if I need to 
	/* create them in the SetMainStage() function. */

	bEjected=false;
	bEVA=false;
	hatch_status=DOOR_CLOSED;
	hatch_proc=0.0;
	oldsimt = -0.01;
	Vold = _V(0,0,0);
	stage = 0;

	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
	RequestLoadVesselWave3(MyID,PLAYGEAR,"Sound\\SPIRAL\\gear.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYBANG,"Sound\\SPIRAL\\stg2sep.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYJETT,"Sound\\SPIRAL\\fasep.wav",BOTHVIEW_FADED_FAR);
	RequestLoadVesselWave3(MyID,PLAYJET,"Sound\\SPIRAL\\jet.wav",BOTHVIEW_FADED_FAR);
	anim_hatch = 0.0;

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();   

	static UINT HatchGrp[1] = {0};
	static MGROUP_ROTATE Hatch (1, HatchGrp, 1,
		_V( 0.0,1.049,1.978), _V(1,0,0), (float)( 155*RAD));
//		_V(-0.422,0.8,6.25), _V(0,0,1), (float)( 130*RAD));

	anim_hatch = CreateAnimation(0);
	AddAnimationComponent (anim_hatch, 0.1, 0.9, &Hatch);

}


SPIRAL::~SPIRAL()
{

}

void SPIRAL::UpdateAnimations()
{	

	// animate top hatch
	if (!bEjected)
		if (hatch_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * HATCH_OPERATING_SPEED;
			if (hatch_status == DOOR_CLOSING) {
				if (hatch_proc > 0.0)
					hatch_proc = max (0.0, hatch_proc-da);
				else {
					hatch_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (hatch_proc < 1.0)
					hatch_proc = min (1.0, hatch_proc+da);
				else {
					hatch_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_hatch, hatch_proc);
		}

}

void SPIRAL::ActivateHatch (DoorStatus action)
{
	hatch_status = action;
}

void SPIRAL::RevertHatch ()
{
	ActivateHatch (hatch_status == DOOR_CLOSED || hatch_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}


void SPIRAL::SeparateEVA ()
{
	VESSELSTATUS vs;
	GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	vs.status=0;
	VECTOR3 ofs = ASTRO_OFS;
	Local2Rel (ofs, vs.rpos);
	char name[256];
	strcpy (name, GetName()); strcat (name, "-PLT");
	oapiCreateVessel (name, "SPIRAL/eva", vs);
}

void SPIRAL::TimeStep(  double simt )
{
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */


	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
			SetMainStage();
	}

	erasecount++;						//  Every once in a while, check for spawned EVA's that are not focus-enabled,
	if (erasecount > 60) {				//  and delete them.
		erasecount = 0;
		char name[256];
			strcpy (name, GetName()); strcat (name, "-PLT");
			OBJHANDLE hvessel=oapiGetVesselByName(name);
			if (hvessel != 0) {
				VESSEL *remv = oapiGetVesselInterface(hvessel);
				if (remv->GetEnableFocus() == false)
					oapiDeleteVessel(hvessel);
			}
	}

// Check to see if the EVA guy is back inside.  If so, start drawing the pilot in his seat again
	if (bEVA) {
		char name[256];
		OBJHANDLE hvessel;
		strcpy (name, GetName()); strcat (name, "-PLT");
		hvessel=oapiGetVesselByName(name);
		if (hvessel == 0) {
			bEVA = false;
			SetMainStage();
		}
	}

	UpdateAnimations();

}

/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */

void SPIRAL::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	SetMainStage(); // this is a good place to ask for the initial setup.
}


void SPIRAL::SetClassCaps(  FILEHANDLE cfg )
{

	CheckCentralMessage();

	SetMainStage();
}

void SPIRAL::SaveState( FILEHANDLE scn )
{
	char cbuf[256];

	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */


//	Undock(ALLDOCKS);

	oapiWriteScenario_int (scn, "CONFIGURATION", stage);
	oapiWriteScenario_int (scn, "EJECTED", bEjected);
	oapiWriteScenario_int (scn, "EVA", bEVA);
	sprintf (cbuf, "%d %0.4f", hatch_status, hatch_proc);
	oapiWriteScenario_string (scn, "HATCH", cbuf);

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void SPIRAL::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		if (!strnicmp (line, "CONFIGURATION", 13)) {
		    sscanf (line+13, "%d", &stage);
		} else if (!strnicmp (line, "EJECTED", 7)) {
			sscanf (line+7, "%d", &bEjected);
		} else if (!strnicmp (line, "EVA", 3)) {
			sscanf (line+3, "%d", &bEVA);
		} else if (!strnicmp (line, "HATCH", 5)) {
		sscanf (line+5, "%d%lf", &hatch_status, &hatch_proc);
		} else ParsePayloadString(line);

		ParseScenarioLineEx (line, vs);
	}

	/* always set the stage whenever something may have changed */
	SetMainStage();

	if (!bEjected) {
		RevertHatch();
		RevertHatch();
	}

}


void SPIRAL::PlaceComponents()
{
	COMMON_MESH tmp;


	if (!bEjected) {
		tmp.MeshHandle(hSeat);
		tmp.MeshPosition(EJSEAT_OFS);
		AddMeshEx (tmp);
		tmp.MeshHandle(hTopHatch);
		tmp.MeshPosition(TOPHATCH_OFS);
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hDummy1);
		tmp.MeshPosition(_V(0,0,0));
		AddMeshEx (tmp);
		tmp.MeshHandle(hDummy2);
		tmp.MeshPosition(_V(0,0,0));
		AddMeshEx (tmp);
	}

	if ((!bEVA) && (!bEjected)) {
		tmp.MeshHandle(hAstro);
		tmp.MeshPosition(ASTRO_OFS);
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hDummy3);
		tmp.MeshPosition(_V(0,0,0));
		AddMeshEx (tmp);
	}
	tmp.MeshHandle(hEjSphere);
	tmp.MeshPosition(_V(0,0,0));
	AddMeshEx (tmp);

}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void SPIRAL::SetMainStage( )
{

	if (stage == 0)
		SetSpiralSphere();
	else
		SetSpiralSphere();
}

void SPIRAL::SetSpiralSphere( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();
	SetSize (3);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass ( SPIRALSPHERE_DRYMASS + CalculateMass() );


	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);
	retex = oapiRegisterReentryTexture("reentry");
	SetReentryTexture(retex,6e6,1,1);

	SetTouchdownPoints (_V(0,-1.0,0.63), _V(-.7,.7,0.63), _V(.7,.7,0.63));
	SetPMI (_V(5.46,6.16,1.22));
	SetCrossSections (_V(2.60,2.60,2.60));
	SetCW (1.0, 1.0, 1.0, 1.0);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (-1e-2);
	SetBankMomentScale (-1e-2);
//	SetLiftCoeffFunc (0); 
//	SetTrimScale (0.05);
	SetCameraOffset(_V(0,0.4,6.9));

	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();

	EnableTransponder(true);


}



/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */


int SPIRAL::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	if (!fPressed) return 0; // only process keydown events
	if (KEYMOD_SHIFT (keystate)) 
	{
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
		if (KEYDOWN (keystate, OAPI_KEY_E))
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				// Report the inclination setpoint
				// Eject Eject Eject!!!!

			if (bEjected) return 1;
			else if (!bEjected) {

				PlayVesselWave3(MyID,PLAYBANG);
				bEjected = true;
				VESSELSTATUS vs;
				GetStatus (vs);
				vs.eng_main = vs.eng_hovr = 0.0;
				vs.fuel=1.0;
				vs.status=0;
				VECTOR3 ofs = TOPHATCH_OFS;
				VECTOR3 vel = _V(0,20.0,-5.0);
				VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				vs.vrot.x = 0.7;
				vs.vrot.y = 0.0;
				vs.vrot.z = 0.0;
				char name[256];
				strcpy (name, GetName()); strcat (name, "-Hatch");
				VESSEL::Create (name, "SPIRAL/SPIRAL_TopHatch", vs);

				GetStatus (vs);
				ofs = ASTRO_OFS;
				vel = _V(0,12.0,0.0);
				rofs = _V(vs.rvel.x, vs.rvel.y, vs.rvel.z);
				rvel = _V(vs.rvel.x, vs.rvel.y, vs.rvel.z);
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				vs.vrot.x = 0.0;
				vs.vrot.y = 0.0;
				vs.vrot.z = 0.0;
	//			char name[256];
				strcpy (name, GetName()); strcat (name, "-Pilot");
				VESSEL::Create (name, "SPIRAL/SpiralPLT", vs);
				OBJHANDLE hvessel;
				hvessel=oapiGetVesselByName(name);
				if (hvessel != 0) 
					oapiSetFocusObject(hvessel);
				SetMainStage();
			}
																
			return 1;
		}
		return 0;
	}
	else { // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_P)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_P, 1.0)) {
				// Toggle the top hatch
				if (!bEjected)
					RevertHatch ();
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) {
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0)) {
				// EVA
				if ((hatch_status == DOOR_OPEN) && (!bEjected))
				{
					char name[256];
					OBJHANDLE hvessel;
					strcpy (name, GetName()); strcat (name, "-PLT");
					hvessel=oapiGetVesselByName(name);
					if (hvessel == 0)
					{
						bEVA = true;
						SeparateEVA();
						SetMainStage();
						hvessel=oapiGetVesselByName(name);
						if (hvessel != 0)
							oapiSetFocusObject(hvessel);
					}
					else
					{
						oapiDeleteVessel(hvessel);
						bEVA = false;
						SetMainStage();
					};
				}

			}
			return 1;
		}

	}
	return 0;
}



int SPIRAL::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}

