/**
*
* $Id: CVEL-Spiral.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

// All data from buran.ru
const double ISP_RB_SL			=     416.0*G; 
const double ISP_RB_VAC			=     449.0*G; //***
const double ISP_RB2_SL			=     436.0*G; 
const double ISP_RB2_VAC		=     445.0*G; //***
const double ISP_ESCAPE_SL		=     221.0*G; 
const double ISP_ESCAPE_VAC		=     245.0*G; //***
const double ISP_SPIRAL_SL		=     320.0*G; 
const double ISP_SPIRAL_VAC		=     320.0*G; //***
const double ISP_JET_SL			=    8500.0*G;
const double ISP_JET_VAC		=   10500.0*G;

const double THRUST_RB_VAC		=   65950.0*G; //***
const double THRUST_RB2_VAC		=    6595.0*G; //***
const double THRUST_ESCAPE_VAC	=	 4140.0*G; //***
const double THRUST_SPIRAL_VAC	=    1500.0*G; //
const double THRUST_JET_VAC		=    2350.0*G; //
const double MAX_ATT_FIRST		=     250.0*G;
const double RB_PROP_MASS		=   38250.0; //*** the total mass of fueled SpiralRB is 45950kg
const double RB2_PROP_MASS		=    4250.0; //*** the total mass of fueled SpiralRB2 is 5170kg
const double ESCAPE_PROP_MASS	=	 1200.0;
const double RCS_PROP_MASS		=	 2500.0;
const double JET_PROP_MASS		=	  500.0;
const double SPIRALSPHERE_DRYMASS		=    490.0; //***
const double ESCAPE_DRYMASS		=    1050.0; //***
const double RB_DRYMASS			=    7700.0; //***
const double RB2_DRYMASS		=     950.0; //***

const double R_EARTH = 6371000;			// radius of the earth (m)
const double U_EARTH = 3.986e5 * 1000 * 1000 * 1000; //( m^3/sec^2)
const double APO_ALT = 240.0;

const VECTOR3 TOPHATCH_OFS		= _V(0,0,0);
const VECTOR3 EJSEAT_OFS		= _V(0.0, 0.10,1.4);
const VECTOR3 ASTRO_OFS			= _V(0.0, 0.25,1.6);

const double HATCH_OPERATING_SPEED = 0.125;

#define PLAYGEAR					 20		// the gear sound
#define PLAYBANG					 24		// the jettison sound
#define PLAYJETT					 25     // entry shield jettison sound
#define PLAYJET						 26     // the jet noise sound

/* make your own vessel class (named SPIRAL here, for example)
   subclassed from COMMON_VESSEL_EX */

class SPIRAL : public COMMON_VESSEL_EX {
public:
	SPIRAL (OBJHANDLE hObj, int fmodel);
	~SPIRAL();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } hatch_status;

	void UpdateAnimations();
	void ActivateHatch (DoorStatus action);
	void RevertHatch();
	void SeparateEVA();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void PlaceComponents();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 
	void SetSpiralSphere();
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);

protected:
	PROPELLANT_HANDLE ph_first,ph_Spiral,ph_escape1,ph_escape2,ph_escape3,ph_Jets;
	THRUSTER_HANDLE   th_first[12];
	THGROUP_HANDLE    thg_first,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hSPIRAL,hSPIRALBay1,hSPIRALBay2,hTopHatch,hEjSphere,hSeat,hAstro,hSoyuzIntx,hSpiralRB,hSpiralRB2,hTailcone,hDummy1,hDummy2,hDummy3,hDummy4;
	SURFHANDLE		  tex,texLH2,texRCS, retex;
	int stage;
	UINT anim_hatch;	// handle for top hatch animation
	double hatch_proc;
	bool bEjected, bEVA;
	VECTOR3 Vold, Vnew;
	double dt, oldsimt;
	int MyID;
	VECTOR3 gpos, grms, pos, dir, rot;
	DWORD i, j;
	double a,e,h,V_periapsis,Altitude;
	VECTOR3 Rel_Vel;
	ANIMATIONCOMPONENT_HANDLE wing1, wing2, wing3, wing4;
	int erasecount;

};




/* basic vessel class setup */

void SPIRAL::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void SPIRAL::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void SPIRAL::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void SPIRAL::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void SPIRAL::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void SPIRAL::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int SPIRAL::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new SPIRAL (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (SPIRAL*)vessel;
}

